@@ -21,17 +21,14 @@
  * the header inclusion statement in your userdefs.h to include
  * one of those with a different name.
  */
-#ifdef QUIT_DEFAULT_YES
-#define REALLY_QUIT "Are you sure you want to quit? [Y] "
+#define REALLY_QUIT_Y "Are you sure you want to quit? [Y] "
 #ifdef VMS
-#define REALLY_EXIT "Really exit from Lynx? [Y] "
+#define REALLY_EXIT_Y "Really exit from Lynx? [Y] "
 #endif /* VMS */
-#else
-#define REALLY_QUIT "Are you sure you want to quit? [N] "
+#define REALLY_QUIT_N "Are you sure you want to quit? [N] "
 #ifdef VMS
-#define REALLY_EXIT "Really exit from Lynx? [N] "
+#define REALLY_EXIT_N "Really exit from Lynx? [N] "
 #endif /* VMS */
-#endif /* QUIT_DEFAULT_YES */
 #define CANCELLED "Cancelled!!!"
 #define CANCELLING "Cancelling!"
 #define NO_CANCEL "Excellent!!!"
@@ -86,13 +83,21 @@
  "DISABLED form reset button.  Use UP or DOWN arrows or tab to move off."
 #define FORM_LINK_OPTION_LIST_MESSAGE \
  "(Option list) Hit return and use arrow keys and return to select option."
+#define CHOICE_LIST_MESSAGE \
+ "(Choice list) Hit return and use arrow keys and return to select option."
 #define FORM_LINK_OPTION_LIST_UNM_MSG \
  "UNMODIFIABLE option list.  Use return or arrow keys to review or leave."
+#define CHOICE_LIST_UNM_MSG \
+ "UNMODIFIABLE choice list.  Use return or arrow keys to review or leave."
 #define NORMAL_LINK_MESSAGE \
  "(NORMAL LINK)   Use right-arrow or <return> to activate."
 #define LINK_NOT_FOUND "The resource requested is not available at this time."
+#define ENTER_LYNX_COMMAND "Enter Lynx keystroke command: "
 #define WWW_WAIT_MESSAGE "Getting %s"
 #define WWW_SKIP_MESSAGE "Skipping %s"
+#define WWW_USING_MESSAGE "Using %s"
+#define WWW_ILLEGAL_URL_MESSAGE "Illegal URL: %s"
+#define WWW_BAD_ADDR_MESSAGE "Badly formed address %s"
 #define ADVANCED_URL_MESSAGE "URL: %s"
 #define WWW_FAIL_MESSAGE "Unable to access WWW file!!!"
 #define WWW_INDEX_MESSAGE "This is a searchable index.  Use %s to search."
@@ -119,6 +124,7 @@
 #define RESETTING_FORM "Resetting form..."
 #define RELOADING_FORM \
  "Reloading document.  Any form entries will be lost!"
+#define CANNOT_TRANSCODE_FORM "Warning: Cannot transcode form data to charset %s!"
 #define BAD_FORM_MAILTO "Malformed mailto form submission!  Cancelled!"
 #define FORM_MAILTO_DISALLOWED "Mail disallowed!  Cannot submit."
 #define FORM_MAILTO_FAILED "Mailto form submission failed!"
@@ -130,6 +136,8 @@
 #define COMMENT_REQUEST_CANCELLED "Comment request cancelled!!!"
 #define INC_ORIG_MSG_PROMPT \
  "Do you wish to include the original message? (y/n) "
+#define INC_PREPARSED_MSG_PROMPT \
+ "Do you wish to include the preparsed source? (y/n) "
 #define SPAWNING_EDITOR_FOR_MAIL \
  "Spawning your selected editor to edit mail message"
 #define ERROR_SPAWNING_EDITOR \
@@ -141,13 +149,20 @@
 #define FILE_SERVED_LINKS_DISALLOWED \
  "file: URLs via served links are disallowed!"
 #define FILE_BOOKMARKS_DISALLOWED "file: URLs via bookmarks are disallowed!"
+#define SPECIAL_VIA_EXTERNAL_DISALLOWED \
+ "This special URL is not allowed in external documents!"
 #define RETURN_TO_LYNX "Press <return> to return to Lynx."
 #ifdef VMS
 #define SPAWNING_MSG \
  "Spawning DCL subprocess.  Use 'logout' to return to Lynx.\n"
 #else
+#ifdef DOSPATH
+#define SPAWNING_MSG \
+ "Type EXIT to return to Lynx.\r\n"
+#else /* UNIX */
 #define SPAWNING_MSG \
  "Spawning your default shell.  Use 'exit' to return to Lynx.\n"
+#endif
 #endif /* VMS */
 #define SPAWNING_DISABLED "Spawning is currently disabled."
 #define DOWNLOAD_DISABLED "The 'd'ownload command is currently disabled."
@@ -157,6 +172,7 @@
 #define NO_DOWNLOAD_PRINT_OP "You cannot download a printing option."
 #define NO_DOWNLOAD_UPLOAD_OP "You cannot download an upload option."
 #define NO_DOWNLOAD_PERMIT_OP "You cannot download an permit option."
+#define NO_DOWNLOAD_SPECIAL "This special URL cannot be downloaded!"
 #define NO_DOWNLOAD_CHOICE "Nothing to download."
 #define TRACE_ON "Trace ON!"
 #define TRACE_OFF "Trace OFF!"
@@ -180,6 +196,7 @@
 #define FORM_ACTION_NOT_HTTP_URL \
  "Sorry, the ACTION for this form is not an http URL."
 #define NOT_HTTP_URL_OR_ACTION "Not an http URL or form ACTION!"
+#define SPECIAL_ACTION_DISALLOWED "This special URL cannot be a form ACTION!"
 #define NOT_IN_STARTING_REALM "URL is not in starting realm!"
 #define NEWSPOSTING_DISABLED "News posting is disabled!"
 #define DIRED_DISABLED "File management support is disabled!"
@@ -246,8 +263,12 @@
  "Minimal comment parsing OFF (Historical is in effect)!"
 #define SOFT_DOUBLE_QUOTE_ON "Soft double-quote parsing ON!"
 #define SOFT_DOUBLE_QUOTE_OFF "Soft double-quote parsing OFF!"
+#define USING_DTD_0 "Now using TagSoup parsing of HTML."
+#define USING_DTD_1 "Now using SortaSGML parsing of HTML!"
 #define ALREADY_AT_END "You are already at the end of this document."
 #define ALREADY_AT_BEGIN "You are already at the beginning of this document."
+#define ALREADY_AT_PAGE "You are already at page %d of this document."
+#define LINK_ALREADY_CURRENT "Link number %d already is current."
 #define ALREADY_AT_FIRST "You are already at the first document"
 #define NO_LINKS_ABOVE "There are no links above this line of the document."
 #define NO_LINKS_BELOW "There are no links below this line of the document."
@@ -284,12 +305,15 @@
 #define GOTO_TELNET_DISALLOWED "You are not allowed to goto \"telnet:\" URLs"
 #define GOTO_TN3270_DISALLOWED "You are not allowed to goto \"tn3270:\" URLs"
 #define GOTO_WAIS_DISALLOWED "You are not allowed to goto \"wais:\" URLs"
+#define GOTO_SPECIAL_DISALLOWED "This special URL is not allowed as a goto!"
 #define URL_TO_OPEN "URL to open: "
 #define EDIT_CURRENT_GOTO "Edit the current Goto URL: "
 #define EDIT_THE_PREV_GOTO "Edit the previous Goto URL: "
 #define EDIT_A_PREV_GOTO "Edit a previous Goto URL: "
 #define CURRENT_DOC_HAS_POST_DATA "Current document has POST data."
 #define EDIT_CURDOC_URL "Edit this document's URL: "
+#define EDIT_CURLINK_URL "Edit the current link's URL: "
+#define EDIT_FM_MENU_URLS_DISALLOWED "You cannot edit File Management URLs"
 #define ENTER_DATABASE_QUERY "Enter a database query: "
 #define ENTER_WHEREIS_QUERY "Enter a whereis query: "
 #define EDIT_CURRENT_QUERY "Edit the current query: "
@@ -323,7 +347,25 @@
 #define PRINT_DISABLED "The 'p'rint command is currently disabled."
 #define NO_TOOLBAR "Document has no Toolbar links or Banner."
 #define NOOPEN_TRAV_ERR_FILE "Unable to open traversal errors output file"
-#define FOLLOW_LINK_NUMBER "Follow link number: "
+#define FOLLOW_LINK_NUMBER "Follow link (or goto link or page) number: "
+#define SELECT_OPTION_NUMBER "Select option (or page) number: "
+#define OPTION_CHOICE_NUMBER "Option choice (or page) number: "
+#define OPTION_ALREADY_CURRENT "Option number %d already is current."
+#define CHOICE_ALREADY_CURRENT "Choice number %d already is current."
+#define ALREADY_AT_OPTION_END \
+ "You are already at the end of this option list."
+#define ALREADY_AT_CHOICE_END \
+ "You are already at the end of this choice list."
+#define ALREADY_AT_OPTION_BEGIN \
+ "You are already at the beginning of this option list."
+#define ALREADY_AT_CHOICE_BEGIN \
+ "You are already at the beginning of this choice list."
+#define ALREADY_AT_OPTION_PAGE \
+ "You are already at page %d of this option list."
+#define ALREADY_AT_CHOICE_PAGE \
+ "You are already at page %d of this choice list."
+#define BAD_OPTION_NUM_ENTERED "You have entered an invalid option number."
+#define BAD_CHOICE_NUM_ENTERED "You have entered an invalid choice number."
 #define BAD_HTML_USE_TRACE "** Bad HTML!!  Use -trace to diagnose. **"
 #define CANNOT_OPEN_TEMP "Can't open temporary file!"
 #define CANNOT_OPEN_OUTPUT "Can't open output file!  Cancelling!"
@@ -363,6 +405,8 @@
 #define PASSWORD_PROMPT "Password: "
 #define USERNAME_PASSWORD_REQUIRED "lynx: Username and Password required!!!"
 #define PASSWORD_REQUIRED "lynx: Password required!!!"
+#define CLEAR_ALL_AUTH_INFO "Clear all authorization info for this session?"
+#define AUTH_INFO_CLEARED "Authorization info cleared."
 #define CGI_DISABLED "cgi support has been disabled by system administrator."
 #define CGI_NOT_COMPILED \
  "Lynxcgi capabilities are not compiled into this version."
@@ -387,6 +431,7 @@
 #define RLOGIN_DISABLED "Rlogin access is disabled!"
 #define FTP_DISABLED "Ftp access is disabled!"
 #define NO_REFS_FROM_DOC "There are no references from this document."
+#define NO_VISIBLE_REFS_FROM_DOC "There are only hidden links from this document."
 #ifdef VMS
 #define CANNOT_OPEN_COMFILE "Unable to open command file."
 #endif /* VMS */
@@ -404,6 +449,8 @@
 #define HAVE_NEW_MAIL_MSG "*** You have new mail. ***"
 #define SAVE_REQUEST_CANCELLED "Save request cancelled!!!"
 #define MAIL_REQUEST_CANCELLED "Mail request cancelled!!!"
+#define CONFIRM_MAIL_SOURCE_PREPARSED \
+ "Viewing preparsed source.  Are you sure you want to mail it? [y]"
 #define MAILING_FILE "Mailing file.  Please wait..."
 #define MAIL_REQUEST_FAILED "ERROR - Unable to mail file"
 #define CONFIRM_LONG_SCREEN_PRINT \
@@ -429,6 +476,7 @@
  "Screen height must be at least 23 lines for the Options menu!"
 #define OPTION_SCREEN_NEEDS_22 \
  "Screen height must be at least 22 lines for the Options menu!"
+#define NEED_ADVANCED_USER_MODE "That key requires Advanced User mode."
 #define COMMAND_PROMPT "Command: "
 #define SELECT_SEGMENT "Select "
 #define CAP_LETT_SEGMENT "capital letter"
@@ -436,9 +484,9 @@
 #define TO_SAVE_SEGMENT " to save,"
 #define OR_SEGMENT " or "
 #define TO_RETURN_SEGMENT " to return to Lynx."
-#define ACCEPT_DATA "Hit return to accept entered data."
+#define ACCEPT_DATA "Hit RETURN to accept entered data."
 #define ACCEPT_DATA_OR_DEFAULT \
-"Hit return to accept entered data.  Delete data to invoke the default."
+"Hit RETURN to accept entered data.  Delete data to invoke the default."
 #define VALUE_ACCEPTED "Value accepted!"
 #define VALUE_ACCEPTED_WARNING_X \
  "Value accepted! -- WARNING: Lynx is configured for XWINDOWS!"
@@ -449,6 +497,8 @@
 #define FAILED_CLEAR_SET_DISPLAY "Failed to clear DISPLAY variable!"
 #define BOOKMARK_CHANGE_DISALLOWED \
  "You are not allowed to change the bookmark file!"
+#define COLOR_TOGGLE_DISABLED "Terminal does not support color"
+#define COLOR_TOGGLE_DISABLED_FOR_TERM "Your '%s' terminal does not support color."
 #define DOTFILE_ACCESS_DISABLED "Access to dot files is disabled!"
 #define UA_COPYRIGHT_WARNING \
  "WARNING: Misrepresentation of the User-Agent may be a copyright violation!"
@@ -460,17 +510,30 @@
 #define R_TO_RETURN_TO_LYNX " 'r' to return to Lynx "
 #define SAVE_OR_R_TO_RETURN_TO_LYNX " '>' to save, or 'r' to return to Lynx "
 #define ANY_KEY_CHANGE_RET_ACCEPT \
- "Hit any key to change value; RETURN to accept: "
+ "Hit any key to change value; RETURN to accept."
 #define ERROR_UNCOMPRESSING_TEMP "Error uncompressing temporary file!"
 #define UNSUPPORTED_URL_SCHEME "Unsupported URL scheme!"
 #define UNSUPPORTED_DATA_URL "Unsupported data: URL!  Use SHOWINFO, for now."
+#define ILLEGAL_REDIRECTION_URL "Illegal redirection URL received from server!"
 #define	SERVER_ASKED_FOR_REDIRECTION \
- "Server asked for redirection of POST content to"
+ "Server asked for %d redirection of POST content to"
 #define	PROCEED_GET_CANCEL "P)roceed, use G)ET or C)ancel "
-#define	ADVANCED_POST_REDIRECT \
+#define	PROCEED_OR_CANCEL "P)roceed, or C)ancel "
+#define	ADVANCED_POST_GET_REDIRECT \
  "Redirection of POST content. P)roceed, see U)RL, use G)ET or C)ancel"
+#define	ADVANCED_POST_REDIRECT \
+ "Redirection of POST content. P)roceed, see U)RL, or C)ancel"
 #define CONFIRM_POST_RESUBMISSION \
  "Document from Form with POST content.  Resubmit?"
+#define CONFIRM_POST_RESUBMISSION_TO \
+ "Resubmit POST content to %s ?"
+#define CONFIRM_POST_LIST_RELOAD \
+ "List from document with POST data.  Reload %s ?"
+#define CONFIRM_POST_DOC_HEAD \
+ "Document from POST action, HEAD may not be understood.  Proceed?"
+#define CONFIRM_POST_LINK_HEAD \
+ "Form submit action is POST, HEAD may not be understood.  Proceed?"
+#define DISCARDING_POST_DATA "Discarding POST data..."
 #define WILL_NOT_RELOAD_DOC "Document will not be reloaded!"
 #define	LOCATION_HEADER "Location: "
 #define STRING_NOT_FOUND "'%s' not found!"
@@ -508,6 +571,8 @@
 #define SENDING_MESSAGE_WITH_BODY_TO \
  "You are sending a message with body to:\n  "
 #define SENDING_COMMENT_TO "You are sending a comment to:\n  "
+#define WITH_COPY_TO "\n With copy to:\n  "
+#define WITH_COPIES_TO "\n With copies to:\n  "
 #define CTRL_G_TO_CANCEL_SEND \
  "\n\nUse Ctrl-G to cancel if you do not want to send a message\n"
 #define ENTER_NAME_OR_BLANK \
@@ -530,6 +595,8 @@
  "\n on a line and press enter again."
 #define ADVANCED_COOKIE_CONFIRMATION \
  "%s cookie: %.*s=%.*s  Allow? (Y/N/Always/neVer)"
+#define INVALID_COOKIE_DOMAIN_CONFIRMATION \
+ "Accept invalid cookie domain=%s for '%s'?"
 #define ALLOWING_COOKIE "Allowing this cookie."
 #define REJECTING_COOKIE "Rejecting this cookie."
 #define COOKIE_JAR_IS_EMPTY "The Cookie Jar is empty."
@@ -560,11 +627,31 @@
 #define ALL_COOKIES_EATEN "All of the cookies in the jar have been eaten!"
 #define PORT_NINETEEN_INVALID "Port 19 not permitted in URLs."
 #define PORT_TWENTYFIVE_INVALID "Port 25 not permitted in URLs."
+#define PORT_INVALID "Port %lu not permitted in URLs."
+#define URL_PORT_BAD "URL has a bad port field."
+#define HTML_STACK_OVERRUN "Maximum nesting of HTML elements exceeded."
+#define BAD_PARTIAL_REFERENCE "Bad partial reference! Stripping lead dots."
+#define TRACELOG_OPEN_FAILED "Trace Log open failed.  Trace off!."
+#define LYNX_TRACELOG_TITLE "Lynx Trace Log"
+#define NO_TRACELOG_STARTED "No trace log has been started for this session."
+#define MAX_TEMPCOUNT_REACHED \
+ "The maximum temporary file count has been reached!"
+#define FORM_VALUE_TOO_LONG \
+ "Form field value exceeds buffer length!  Trim the tail."
+#define FORM_TAIL_COMBINED_WITH_HEAD \
+ "Modified tail combined with head of form field value."
 
 #ifdef DIRED_SUPPORT
 #define DIRED_NOVICELINE \
  "  C)reate  D)ownload  E)dit  F)ull menu  M)odify  R)emove  T)ag  U)pload     \n"
 #define CURRENT_LINK_STATUS_FAILED "Failed to obtain status of current link!"
+
+#define INVALID_PERMIT_URL \
+ "Special URL only valid from current File Permission menu!"
 #endif /* DIRED_SUPPORT */
 
+#ifdef USE_EXTERNALS
+#define EXTERNALS_DISABLED "External support is currently disabled."
+#endif /* USE_EXTERNALS */
+
 #endif /* LYMESSAGES_EN_H */
