@@ -17,6 +17,9 @@
 #include "tcp.h"
 #include "HTFormat.h"
 #include "HTStream.h"
+#include "UCDefs.h"
+#include "UCMap.h"
+#include "UCAux.h"
 #include "HTVMSUtils.h"
 /*#include <stdio.h> included by HTUtils.h -- FM */
 /*#include <unixlib.h> included by HTUtils.h -- FM */
@@ -32,6 +35,8 @@
 
 #include "LYLeaks.h"
 
+#define FREE(x) if (x) {free(x); x = NULL;}
+
 #define INFINITY 512            	/* File name length @@ FIXME */
 
 PUBLIC BOOL HTVMSFileVersions=FALSE; /* Include version numbers in listing? */
@@ -367,8 +372,8 @@ PUBLIC char * HTVMS_name ARGS2(
 	for (p=strchr(vmsname, '['); *p!=']'; p++)
 	    if (*p=='/') *p='.';	/* Convert dir sep.  to dots */
     }
-    free(nodename);
-    free(filename);
+    FREE(nodename);
+    FREE(filename);
     return vmsname;
 }
 
@@ -475,7 +480,7 @@ char Name[256];
 }
 
 /*** "dirent.h" ***/
-#include <types.h>
+/* #include <types.h>	already in tcp.h */
 
 #ifndef	_POSIX_SOURCE
 #define	d_ino	d_fileno	/* compatability */
@@ -498,9 +503,9 @@ typedef	struct __dirdesc {
 	struct dsc$descriptor_s dirname_desc;	/* descriptor of dirname */
 } DIR;
 
-extern	DIR *opendir(char *dirname);
-extern	struct dirent *readdir(DIR *dirp);
-extern	int closedir(DIR *dirp);
+PRIVATE	DIR *HTVMSopendir(char *dirname);
+PRIVATE	struct dirent *HTVMSreaddir(DIR *dirp);
+PRIVATE	int HTVMSclosedir(DIR *dirp);
 #if 0
 #ifndef	_POSIX_SOURCE
 extern	void seekdir(/* DIR *dirp, int loc */);
@@ -546,7 +551,7 @@ struct	dirent {
 #endif	/* !_POSIX_SOURCE */
 
 
-DIR *opendir(char *dirname)
+PRIVATE DIR *HTVMSopendir(char *dirname)
 {
 static DIR dir;
 char *closebracket;
@@ -643,7 +648,7 @@ char *dot;
    return(&dir);
 }
 
-struct dirent *readdir(DIR *dirp)
+PRIVATE struct dirent *HTVMSreaddir(DIR *dirp)
 {
 static struct dirent entry;
 long status;
@@ -685,7 +690,7 @@ char *UnixEntry;
    }
 }
 
-int closedir(DIR *dirp)
+PRIVATE int HTVMSclosedir(DIR *dirp)
 {
 long status;
 
@@ -700,15 +705,14 @@ long status;
 #include "HTBTree.h"
 #include "HTFile.h"	/* For HTFileFormat() */
 #include "HTAlert.h"
-#define FREE(x) if (x) {free(x); x=NULL;}
 /*
 **  Hypertext object building machinery.
 */
 #include "HTML.h"
 #define PUTC(c) (*targetClass.put_character)(target, c)
 #define PUTS(s) (*targetClass.put_string)(target, s)
-#define START(e) (*targetClass.start_element)(target, e, 0, 0)
-#define END(e) (*targetClass.end_element)(target, e)
+#define START(e) (*targetClass.start_element)(target, e, 0, 0, -1, 0)
+#define END(e) (*targetClass.end_element)(target, e, 0)
 #define FREE_TARGET (*targetClass._free)(target)
 #define ABORT_TARGET (*targetClass._free)(target)
 struct _HTStructured {
@@ -732,10 +736,10 @@ typedef struct _VMSEntryInfo {
 
 PRIVATE void free_VMSEntryInfo_struct_contents ARGS1(VMSEntryInfo *,entry_info)
 {
-    if(entry_info) {
-        if(entry_info->filename) free(entry_info->filename);
-        if(entry_info->type) free(entry_info->type);
-        if(entry_info->date) free(entry_info->date);
+    if (entry_info) {
+	FREE(entry_info->filename);
+	FREE(entry_info->type);
+	FREE(entry_info->date);
     }
    /* dont free the struct */
 }
@@ -749,7 +753,8 @@ extern BOOLEAN HTfileSortMethod;  /* specifies the method of sorting */
 PUBLIC int compare_VMSEntryInfo_structs ARGS2(VMSEntryInfo *,entry1, 
 					      VMSEntryInfo *,entry2)
 {
-    int status;
+    int i, status;
+    char date1[16], date2[16], time1[8], time2[8], month[4];
 
     switch(HTfileSortMethod)
       {
@@ -757,7 +762,7 @@ PUBLIC int compare_VMSEntryInfo_structs ARGS2(VMSEntryInfo *,entry1,
 			/* both equal or both 0 */
                         if(entry1->size == entry2->size)
 			    return(strcasecomp(entry1->filename, 
-							entry2->filename));
+					       entry2->filename));
 			else
 			    if(entry1->size > entry2->size)
 				return(1);
@@ -772,23 +777,83 @@ PUBLIC int compare_VMSEntryInfo_structs ARGS2(VMSEntryInfo *,entry1,
 			    /* else fall to filename comparison */
 			}
                         return (strcasecomp(entry1->filename, 
-							entry2->filename));
+					    entry2->filename));
                         break;
         case FILE_BY_DATE:
                         if(entry1->date && entry2->date) {
-                                /* We really should change the type :( */
-                            status = strcasecomp(entry1->date, entry2->date);
+			    /*
+			    ** Make sure we have the correct length. - FM
+			    */
+			    if (strlen(entry1->date) != 12 ||
+			        strlen(entry2->date) != 12) {
+				return (strcasecomp(entry1->filename, 
+						    entry2->filename));
+			    }
+			    /*
+			    ** Set up for sorting in reverse
+			    ** chronological order. - FM
+			    */
+			    if (entry1->date[7] != ' ') {
+			        strcpy(date1, "9999");
+				strcpy(time1, (char *)&entry1->date[7]);
+			    } else {
+				strcpy(date1, (char *)&entry1->date[8]);
+			        strcpy(time1, "00:00");
+			    }
+			    strncpy(month, entry1->date, 3);
+			    month[3] = '\0';
+			    for (i = 0; i < 12; i++) {
+			        if (!strcasecomp(month, months[i])) {
+				    break;
+				}
+			    }
+			    i++;
+			    sprintf(month, "%s%d", (i < 10 ? "0" : ""), i);
+			    strcat(date1, month);
+			    strncat(date1, (char *)&entry1->date[4], 2);
+			    date1[8] = '\0';
+			    if (date1[6] == ' ') {
+			        date1[6] = '0';
+			    }
+			    strcat(date1, time1);
+			    if (entry2->date[7] != ' ') {
+			        strcpy(date2, "9999");
+				strcpy(time2, (char *)&entry2->date[7]);
+			    } else {
+				strcpy(date2, (char *)&entry2->date[8]);
+			        strcpy(time2, "00:00");
+			    }
+			    strncpy(month, entry2->date, 3);
+			    month[3] = '\0';
+			    for (i = 0; i < 12; i++) {
+			        if (!strcasecomp(month, months[i])) {
+				    break;
+				}
+			    }
+			    i++;
+			    sprintf(month, "%s%d", (i < 10 ? "0" : ""), i);
+			    strcat(date2, month);
+			    strncat(date2, (char *)&entry2->date[4], 2);
+			    date2[8] = '\0';
+			    if (date2[6] == ' ') {
+			        date2[6] = '0';
+			    }
+			    strcat(date2, time2);
+			    /*
+			    ** Do the comparison. - FM
+			    */
+                            status = strcasecomp(date2, date1);
 			    if(status)
 				return(status);
 			    /* else fall to filename comparison */
 			}
-                        return (strcasecomp(entry1->filename, 
-							entry2->filename));
+                        return (strcasecomp(entry1->filename,
+					    entry2->filename));
                         break;
         case FILE_BY_NAME:
         default:
-                        return (strcasecomp(entry1->filename, 
-							entry2->filename));
+                        return (strcmp(entry1->filename, 
+					    entry2->filename));
       }
 }
 
@@ -825,15 +890,16 @@ PUBLIC int HTVMSBrowseDir ARGS4(
     char *parent = NULL;
     char *relative = NULL;
     char *cp, *cp1;
-    int  pathend;
+    int  pathend, len;
     DIR *dp;
     struct stat file_info;
     time_t NowTime;
     static char ThisYear[8];
     VMSEntryInfo *entry_info=0;
     char string_buffer[64];
-    extern BOOLEAN nodotfiles;
+    extern BOOLEAN no_dotfiles, show_dotfiles;
 
+    HTUnEscape(pathname);
     CTRACE(stderr,"HTVMSBrowseDir: Browsing `%s\'\n", pathname);
 
     /*
@@ -846,7 +912,7 @@ PUBLIC int HTVMSBrowseDir ARGS4(
     	 (cp=strchr(pathname+1, '/')) == NULL ||
 	 *(cp+1) == '\0' ||
 	 0==strncmp((cp+1), "000000", 6)) ||
-        (dp=opendir(pathname)) == NULL) {
+        (dp=HTVMSopendir(pathname)) == NULL) {
         FREE(pathname);
     	return HTLoadError(sink, 403, "Could not access directory.");
     }
@@ -855,6 +921,12 @@ PUBLIC int HTVMSBrowseDir ARGS4(
      *  Set up the output stream.
      */
     _HTProgress ("Building directory listing...");
+    if (UCLYhndl_HTFile_for_unspec >= 0) {
+	HTAnchor_setUCInfoStage(anchor,
+				UCLYhndl_HTFile_for_unspec,
+				UCT_STAGE_PARSER,
+				UCT_SETBY_DEFAULT);
+    }
     target = HTML_new(anchor, format_out, sink);
     targetClass = *(target->isa);
 
@@ -921,7 +993,9 @@ PUBLIC int HTVMSBrowseDir ARGS4(
 	    START(HTML_PRE);
 	    for(;;) {
 	        char c = fgetc(fp);
-	        if (c == (char)EOF) break;
+	        if (c == (char)EOF)
+		    break;
+#ifdef NOTDEFINED
 	        switch (c) {
 	    	    case '&':
 		    case '<':
@@ -935,6 +1009,9 @@ PUBLIC int HTVMSBrowseDir ARGS4(
 		    default:
 			PUTC(c);
 	        }
+#else
+		PUTC(c);
+#endif /* NOTDEFINED */
 	    }
 	    END(HTML_PRE);
 	    fclose(fp);
@@ -976,7 +1053,7 @@ PUBLIC int HTVMSBrowseDir ARGS4(
 	bt = HTBTree_new((HTComparer) compare_VMSEntryInfo_structs);
 
 	/* Build tree */
-	while ((dirbuf = readdir(dp))) {
+	while ((dirbuf = HTVMSreaddir(dp))) {
 	    HTAtom *encoding = NULL;
 	    HTFormat format;
 
@@ -999,7 +1076,7 @@ PUBLIC int HTVMSBrowseDir ARGS4(
 	    }
 
 	    /* Skip files beginning with a dot? */
-	    if (nodotfiles && *dirbuf->d_name == '.') {
+	    if ((no_dotfiles || !show_dotfiles) && *dirbuf->d_name == '.') {
 		continue;
 	    }
 
@@ -1021,15 +1098,18 @@ PUBLIC int HTVMSBrowseDir ARGS4(
 	    entry_info->display = TRUE;
 
 	    /* Get the type */
-	    format = HTFileFormat(dirbuf->d_name, &encoding);
-	    if(!strncmp(HTAtom_name(format), "application",11)) 
-	      {
-		   cp = HTAtom_name(format) + 12;
-		   if(!strncmp(cp,"x-",2))
-			cp+=2;
-	      }
-	    else
-		cp = HTAtom_name(format);
+	    format = HTFileFormat(dirbuf->d_name, &encoding,
+				  (CONST char **)&cp);
+	    if (!cp) {
+		if(!strncmp(HTAtom_name(format), "application",11)) 
+		{
+		    cp = HTAtom_name(format) + 12;
+		    if(!strncmp(cp,"x-", 2))
+			cp += 2;
+		}
+		else
+		    cp = HTAtom_name(format);
+	    }
 	    StrAllocCopy(entry_info->type, cp);
 
 	    StrAllocCopy(entry_info->filename, dirbuf->d_name);
@@ -1039,7 +1119,39 @@ PUBLIC int HTVMSBrowseDir ARGS4(
                 dot = strstr(entry_info->filename, ".DIR");
                 if (dot)
                    *dot = '\0';
+		cp = entry_info->filename;
+		while (cp && *cp) {
+		    *cp = TOLOWER(*cp);
+		    cp++;
+		}
 		StrAllocCopy(entry_info->type, "Directory");
+	    } else {
+	        if ((cp = strstr(entry_info->filename, "READ")) == NULL) {
+	            cp = entry_info->filename;
+		} else {
+		    cp += 4;
+		    if (!strncmp(cp, "ME", 2)) {
+		        cp += 2;
+			while (cp && *cp && *cp != '.') {
+			    cp++;
+			}
+		    } else if (!strncmp(cp, ".ME", 3)) {
+		        cp = (entry_info->filename +
+			      strlen(entry_info->filename));
+		    } else {
+		        cp = entry_info->filename;
+		    }
+		}
+		while (cp && *cp) {
+		    *cp = TOLOWER(*cp);
+		    cp++;
+		}
+		if (((len = strlen(entry_info->filename)) > 2) &&
+		    entry_info->filename[len-1] == 'z') {
+		    if (entry_info->filename[len-2] == '.' ||
+		        entry_info->filename[len-2] == '_')
+			entry_info->filename[len-1] = 'Z';
+		}
 	    }
 
 	    /* Get the date */
@@ -1071,10 +1183,10 @@ PUBLIC int HTVMSBrowseDir ARGS4(
 	         HTBTree_add(bt, (VMSEntryInfo *)entry_info); 
 	      }
 
-	} /* End while readdir() */
+	} /* End while HTVMSreaddir() */
 
 	FREE(pathname);
-	closedir(dp);
+	HTVMSclosedir(dp);
 
 	START(HTML_PRE);
 	/*
