@@ -46,11 +46,11 @@ PUBLIC void HTFormatInit NOARGS
 {
  FILE *fp = NULL;
  char *cp = NULL;
- extern char * personal_type_map, * global_type_map;
 
 #ifdef NeXT
   HTSetPresentation("application/postscript",   "open %s", 1.0, 2.0, 0.0, 0);
-  HTSetPresentation("image/x-tiff",             "open %s", 1.0, 2.0, 0.0, 0);
+  HTSetPresentation("image/x-tiff",             "open %s", 2.0, 2.0, 0.0, 0);
+  HTSetPresentation("image/tiff",               "open %s", 1.0, 2.0, 0.0, 0);
   HTSetPresentation("audio/basic",              "open %s", 1.0, 2.0, 0.0, 0);
   HTSetPresentation("*",                        "open %s", 1.0, 0.0, 0.0, 0);
 #else
@@ -60,9 +60,11 @@ PUBLIC void HTFormatInit NOARGS
   HTSetPresentation("image/gif",        XLoadImageCommand,  1.0, 3.0, 0.0, 0);
   HTSetPresentation("image/x-xbm",      XLoadImageCommand,  1.0, 3.0, 0.0, 0);
   HTSetPresentation("image/x-xbitmap",  XLoadImageCommand,  1.0, 3.0, 0.0, 0);
-  HTSetPresentation("image/x-png",      XLoadImageCommand,  1.0, 3.0, 0.0, 0);
+  HTSetPresentation("image/x-png",      XLoadImageCommand,  2.0, 3.0, 0.0, 0);
+  HTSetPresentation("image/png",	XLoadImageCommand,  1.0, 3.0, 0.0, 0);
   HTSetPresentation("image/x-rgb",      XLoadImageCommand,  1.0, 3.0, 0.0, 0);
-  HTSetPresentation("image/x-tiff",     XLoadImageCommand,  1.0, 3.0, 0.0, 0);
+  HTSetPresentation("image/x-tiff",     XLoadImageCommand,  2.0, 3.0, 0.0, 0);
+  HTSetPresentation("image/tiff",	XLoadImageCommand,  1.0, 3.0, 0.0, 0);
   HTSetPresentation("image/jpeg",       XLoadImageCommand,  1.0, 3.0, 0.0, 0);
   HTSetPresentation("video/mpeg",       "mpeg_play %s &",   1.0, 3.0, 0.0, 0);
 
@@ -110,16 +112,16 @@ PUBLIC void HTFormatInit NOARGS
  HTSetConversion("application/html", "text/plain",
  					HTMLToPlain,	0.5, 0.0, 0.0, 0);
  HTSetConversion("application/html", "www/present",
- 					HTMLPresent,	1.0, 0.0, 0.0, 0);
+ 					HTMLPresent,	2.0, 0.0, 0.0, 0);
  HTSetConversion("application/html", "www/source",
  					HTPlainPresent,	1.0, 0.0, 0.0, 0);
- HTSetConversion("application/x-wais-source", "www/source",  	     
+ HTSetConversion("application/x-wais-source", "www/source",
 					HTPlainPresent,	1.0, 0.0, 0.0, 0);
- HTSetConversion("application/x-wais-source", "www/present",  	     
-				        HTWSRCConvert,	1.0, 0.0, 0.0, 0);
- HTSetConversion("application/x-wais-source", "www/download",  	     
+ HTSetConversion("application/x-wais-source", "www/present",
+				        HTWSRCConvert,	2.0, 0.0, 0.0, 0);
+ HTSetConversion("application/x-wais-source", "www/download",
 					HTWSRCConvert,	1.0, 0.0, 0.0, 0);
- HTSetConversion("application/x-wais-source", "www/dump",  	     
+ HTSetConversion("application/x-wais-source", "www/dump",
 					HTWSRCConvert,	1.0, 0.0, 0.0, 0);
 
  /*
@@ -145,10 +147,11 @@ PUBLIC void HTFormatInit NOARGS
  HTSetConversion("text/x-sgml",
  			      "www/source",  HTPlainPresent, 1.0, 0.0, 0.0, 0);
  HTSetConversion("text/x-sgml",
- 			      "www/present", HTMLPresent,    1.0, 0.0, 0.0, 0);
+ 			      "www/present", HTMLPresent,    2.0, 0.0, 0.0, 0);
  HTSetConversion("text/sgml", "www/source",  HTPlainPresent, 1.0, 0.0, 0.0, 0);
  HTSetConversion("text/sgml", "www/present", HTMLPresent,    1.0, 0.0, 0.0, 0);
  HTSetConversion("text/plain","www/present", HTPlainPresent, 1.0, 0.0, 0.0, 0);
+ HTSetConversion("text/plain","www/source",  HTPlainPresent, 1.0, 0.0, 0.0, 0);
  HTSetConversion("text/html", "www/source",  HTPlainPresent, 1.0, 0.0, 0.0, 0);
  HTSetConversion("text/html", "text/x-c",    HTMLToC, 	     0.5, 0.0, 0.0, 0);
  HTSetConversion("text/html", "text/plain",  HTMLToPlain,    0.5, 0.0, 0.0, 0);
@@ -177,6 +180,19 @@ PUBLIC void HTFormatInit NOARGS
      HTLoadTypesConfigFile(buffer);
  }
 
+ /*
+  *  Put text/html and text/plain at beginning of list. - kw
+  */
+ HTReorderPresentation(WWW_PLAINTEXT, WWW_PRESENT);
+ HTReorderPresentation(WWW_HTML, WWW_PRESENT);
+}
+
+PUBLIC void HTPreparsedFormatInit NOARGS
+{
+ if (LYPreparsedSource) {
+     HTSetConversion("text/html", "www/source", HTMLParsedPresent, 1.0, 0.0, 0.0, 0);
+     HTSetConversion("text/html", "www/dump",	HTMLParsedPresent, 1.0, 0.0, 0.0, 0);
+ }
 }
 
 /* Some of the following is taken from: */
@@ -184,19 +200,19 @@ PUBLIC void HTFormatInit NOARGS
 /*
 Copyright (c) 1991 Bell Communications Research, Inc. (Bellcore)
 
-Permission to use, copy, modify, and distribute this material 
-for any purpose and without fee is hereby granted, provided 
-that the above copyright notice and this permission notice 
-appear in all copies, and that the name of Bellcore not be 
-used in advertising or publicity pertaining to this 
-material without the specific, prior written permission 
-of an authorized representative of Bellcore.  BELLCORE 
-MAKES NO REPRESENTATIONS ABOUT THE ACCURACY OR SUITABILITY 
-OF THIS MATERIAL FOR ANY PURPOSE.  IT IS PROVIDED "AS IS", 
+Permission to use, copy, modify, and distribute this material
+for any purpose and without fee is hereby granted, provided
+that the above copyright notice and this permission notice
+appear in all copies, and that the name of Bellcore not be
+used in advertising or publicity pertaining to this
+material without the specific, prior written permission
+of an authorized representative of Bellcore.  BELLCORE
+MAKES NO REPRESENTATIONS ABOUT THE ACCURACY OR SUITABILITY
+OF THIS MATERIAL FOR ANY PURPOSE.  IT IS PROVIDED "AS IS",
 WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES.
 */
-/****************************************************** 
-    Metamail -- A tool to help diverse mail readers 
+/******************************************************
+    Metamail -- A tool to help diverse mail readers
                 cope with diverse multimedia mail formats.
 
     Author:  Nathaniel S. Borenstein, Bellcore
@@ -220,29 +236,31 @@ struct MailcapEntry {
 PRIVATE int ExitWithError PARAMS((char *txt));
 PRIVATE int PassesTest PARAMS((struct MailcapEntry *mc));
 
-#define LINE_BUF_SIZE       2000
-#define TMPFILE_NAME_SIZE 127
+#define LINE_BUF_SIZE		2048
+#define TMPFILE_NAME_SIZE	256
 
-PRIVATE char *GetCommand ARGS2(char *,s, char **,t)
+PRIVATE char *GetCommand ARGS2(
+	char *,		s,
+	char **,	t)
 {
     char *s2;
     int quoted = 0;
 
     /* marca -- added + 1 for error case -- oct 24, 1993. */
     s2 = malloc(strlen(s)*2 + 1); /* absolute max, if all % signs */
-    if (!s2) 
-        ExitWithError("Out of memory");
+    if (!s2)
+	ExitWithError("Out of memory");
 
     *t = s2;
     while (s && *s) {
 	if (quoted) {
-            if (*s == '%') *s2++ = '%'; /* Quote through next level, ugh! */
+	    if (*s == '%') *s2++ = '%'; /* Quote through next level, ugh! */
 
-            *s2++ = *s++;
+	    *s2++ = *s++;
 	    quoted = 0;
 	} else {
 	    if (*s == ';') {
-                *s2 = 0;
+		*s2 = '\0';
 		return(++s);
 	    }
 	    if (*s == '\\') {
@@ -253,49 +271,53 @@ PRIVATE char *GetCommand ARGS2(char *,s, char **,t)
 	    }
 	}
     }
-    *s2 = 0;
+    *s2 = '\0';
     return(NULL);
-}	
+}
 
 /* no leading or trailing space, all lower case */
-PRIVATE char *Cleanse ARGS1(char *,s) 
+PRIVATE char *Cleanse ARGS1(
+	char *,		s)
 {
     char *tmp, *news;
-    
+
     /* strip leading white space */
     while (*s && isspace((unsigned char) *s))
-        ++s;
+	++s;
     news = s;
     /* put in lower case */
     for (tmp=s; *tmp; ++tmp) {
-        *tmp = TOLOWER ((unsigned char)*tmp);
+	*tmp = TOLOWER ((unsigned char)*tmp);
     }
     /* strip trailing white space */
-    while (*--tmp && isspace((unsigned char) *tmp))
-        *tmp = 0;
+    while ((tmp > news) && *--tmp && isspace((unsigned char) *tmp))
+	*tmp = '\0';
     return(news);
 }
 
-PRIVATE int ProcessMailcapEntry ARGS2(FILE *,fp, struct MailcapEntry *,mc)
+PRIVATE int ProcessMailcapEntry ARGS2(
+	FILE *,			fp,
+	struct MailcapEntry *,	mc)
 {
-    int i, j, rawentryalloc = 2000, len;
+    int i, j;
+    size_t rawentryalloc = 2000, len;
     char *rawentry, *s, *t, *LineBuf;
 
-    LineBuf = malloc(LINE_BUF_SIZE);
-    if (!LineBuf) 
-        ExitWithError("Out of memory");
-    rawentry = malloc(1 + rawentryalloc);
-    if (!rawentry) 
-        ExitWithError("Out of memory");
-    *rawentry = 0;
+    LineBuf = (char *)malloc(LINE_BUF_SIZE);
+    if (!LineBuf)
+	ExitWithError("Out of memory");
+    rawentry = (char *)malloc(1 + rawentryalloc);
+    if (!rawentry)
+	ExitWithError("Out of memory");
+    *rawentry = '\0';
     while (fgets(LineBuf, LINE_BUF_SIZE, fp)) {
-	if (LineBuf[0] == '#') 
+	if (LineBuf[0] == '#')
 	    continue;
-        len = strlen(LineBuf);
-        if (len == 0) 
+	len = strlen(LineBuf);
+	if (len == 0)
 	    continue;
-        if (LineBuf[len-1] == '\n') 
-	    LineBuf[--len] = 0;
+	if (LineBuf[len-1] == '\n')
+	    LineBuf[--len] = '\0';
 	if ((len + strlen(rawentry)) > rawentryalloc) {
 	    rawentryalloc += 2000;
 	    rawentry = realloc(rawentry, rawentryalloc+1);
@@ -303,7 +325,7 @@ PRIVATE int ProcessMailcapEntry ARGS2(FILE *,fp, struct MailcapEntry *,mc)
 	        ExitWithError("Out of memory");
 	}
 	if (len > 0 && LineBuf[len-1] == '\\') {
-            LineBuf[len-1] = 0;
+	    LineBuf[len-1] = '\0';
 	    strcat(rawentry, LineBuf);
 	} else {
 	    strcat(rawentry, LineBuf);
@@ -314,7 +336,7 @@ PRIVATE int ProcessMailcapEntry ARGS2(FILE *,fp, struct MailcapEntry *,mc)
     FREE(LineBuf);
 
     for (s = rawentry; *s && isspace((unsigned char) *s); ++s)
-        ;
+	;
     if (!*s) {
 	/* totally blank entry -- quietly ignore */
 	FREE(rawentry);
@@ -324,27 +346,27 @@ PRIVATE int ProcessMailcapEntry ARGS2(FILE *,fp, struct MailcapEntry *,mc)
     if (s == NULL) {
 	if (TRACE) {
 		fprintf(stderr,
-			"metamail: Ignoring invalid mailcap entry: %s\n",
+		 "ProcessMailcapEntry: Ignoring invalid mailcap entry: %s\n",
 			rawentry);
 	}
 	FREE(rawentry);
 	return(0);
     }
-    *s++ = 0;
+    *s++ = '\0';
     if (!strncasecomp(rawentry, "text/html", 9) ||
-        !strncasecomp(rawentry, "text/plain", 10)) {
+	!strncasecomp(rawentry, "text/plain", 10)) {
 	--s;
 	*s = ';';
 	if (TRACE) {
 		fprintf(stderr,
-			"metamail: Ignoring mailcap entry: %s\n",
+			"ProcessMailcapEntry: Ignoring mailcap entry: %s\n",
 			rawentry);
 	}
 	FREE(rawentry);
 	return(0);
     }
     for (i = 0, j = 0; rawentry[i]; i++) {
-        if (rawentry[i] != ' ') {
+	if (rawentry[i] != ' ') {
 	    rawentry[j++] = TOLOWER(rawentry[i]);
 	}
     }
@@ -355,25 +377,27 @@ PRIVATE int ProcessMailcapEntry ARGS2(FILE *,fp, struct MailcapEntry *,mc)
     mc->testcommand = NULL;
     mc->label = NULL;
     mc->printcommand = NULL;
-    mc->contenttype = malloc(1+strlen(rawentry));
+    mc->contenttype = (char *)malloc(1 + strlen(rawentry));
     if (!mc->contenttype)
-        ExitWithError("Out of memory");
+	ExitWithError("Out of memory");
     strcpy(mc->contenttype, rawentry);
     mc->quality = 1.0;
     mc->maxbytes = 0;
     t = GetCommand(s, &mc->command);
     if (!t) {
-        goto assign_presentation;
+	goto assign_presentation;
     }
     s = t;
     while (s && *s && isspace((unsigned char) *s)) ++s;
     while (s) {
 	char *arg, *eq, *mallocd_string;
 
-        t = GetCommand(s, &mallocd_string);
+	t = GetCommand(s, &mallocd_string);
 	arg = mallocd_string;
-        eq = strchr(arg, '=');
-        if (eq) *eq++ = 0;
+	eq = strchr(arg, '=');
+	if (eq) {
+	    *eq++ = '\0';
+	}
 	if (arg && *arg) {
 	    arg = Cleanse(arg);
 	    if (!strcmp(arg, "needsterminal")) {
@@ -385,11 +409,11 @@ PRIVATE int ProcessMailcapEntry ARGS2(FILE *,fp, struct MailcapEntry *,mc)
 		StrAllocCopy(mc->testcommand, eq);
 		if (TRACE)
 		    fprintf(stderr,
-		    	    "[HTInit]: found testcommand:%s\n",
+		    	    "ProcessMailcapEntry: Found testcommand:%s\n",
 			    mc->testcommand);
 	    } else if (eq && !strcmp(arg, "description")) {
 		mc->label = eq;
-	    } else if (eq && !strcmp(arg, "label")) { 
+	    } else if (eq && !strcmp(arg, "label")) {
 		mc->label = eq; /* bogus old name for description */
 	    } else if (eq && !strcmp(arg, "print")) {
 		mc->printcommand = eq;
@@ -407,7 +431,8 @@ PRIVATE int ProcessMailcapEntry ARGS2(FILE *,fp, struct MailcapEntry *,mc)
 	    } else if (strcmp(arg, "notes")) { /* IGNORE notes field */
 		if (*arg && TRACE)
 		    fprintf(stderr,
-		    	    "metamail: Ignoring mailcap flag: %s\n", arg);
+			"ProcessMailcapEntry: Ignoring mailcap flag '%s'.\n",
+			    arg);
 	    }
 
 	}
@@ -421,7 +446,7 @@ assign_presentation:
     if (PassesTest(mc)) {
 	if (TRACE)
 	    fprintf(stderr,
-	    	    "[HTInit] Setting up conversion %s : %s\n",
+	    	    "ProcessMailcapEntry Setting up conversion %s : %s\n",
 		    mc->contenttype, mc->command);
 	HTSetPresentation(mc->contenttype, mc->command,
 			  mc->quality, 3.0, 0.0, mc->maxbytes);
@@ -432,50 +457,79 @@ assign_presentation:
     return(1);
 }
 
-PRIVATE void BuildCommand ARGS3(char *,Buf, char *,controlstring, 
-							char *,TmpFileName)
+PRIVATE void BuildCommand ARGS5(
+	char **, 	pBuf,
+	size_t,		Bufsize,
+	char *,		controlstring,
+	char *,		TmpFileName,
+	size_t,		TmpFileLen)
 {
     char *from, *to;
-    int prefixed = 0; 
+    int prefixed = 0;
 
-    for (from = controlstring, to = Buf; *from != '\0'; from++) {
-        if (prefixed) {
-            prefixed = 0;
-            switch(*from) {
-                case '%':
-                    *to++ = '%';
-                    break;
+    for (from = controlstring, to = *pBuf; *from != '\0'; from++) {
+	if (prefixed) {
+	    prefixed = 0;
+	    switch(*from) {
+		case '%':
+		    *to++ = '%';
+		    break;
 		case 'n':
 		case 'F':
 		    if (TRACE) {
 		        fprintf(stderr,
-				"metamail: Bad mailcap \"test\" clause: %s\n",
+			     "BuildCommand: Bad mailcap \"test\" clause: %s\n",
 				controlstring);
 		    }
 		case 's':
-                    if (TmpFileName) {
-                        strcpy(to, TmpFileName);
-                        to += strlen(TmpFileName);
-                    }
-                    break;
-                default:
+		    if (TmpFileLen && TmpFileName) {
+			if ((to - *pBuf) + TmpFileLen + 1 > Bufsize) {
+			    *to = '\0';
+			    if (TRACE) {
+				fprintf(stderr,
+			"BuildCommand: Too long mailcap \"test\" clause,\n");
+				fprintf(stderr,
+					"              ignoring: %s%s...\n",
+					*pBuf, TmpFileName);
+			    }
+			    **pBuf = '\0';
+			    return;
+			}
+			strcpy(to, TmpFileName);
+			to += strlen(TmpFileName);
+		    }
+		    break;
+		default:
 		    if (TRACE) {
-                    	fprintf(stderr,
-		"Ignoring unrecognized format code in mailcap file: %%%c\n",
+			fprintf(stderr,
+  "BuildCommand: Ignoring unrecognized format code in mailcap file '%%%c'.\n",
 			*from);
 		    }
-                    break;
-            }
-        } else if (*from == '%') {
-            prefixed = 1;
-        } else {
-            *to++ = *from;
-        }
+		    break;
+	    }
+	} else if (*from == '%') {
+	    prefixed = 1;
+	} else {
+	    *to++ = *from;
+	}
+	if (to >= *pBuf + Bufsize) {
+	    (*pBuf)[Bufsize - 1] = '\0';
+	    if (TRACE) {
+		fprintf(stderr,
+			"BuildCommand: Too long mailcap \"test\" clause,\n");
+		fprintf(stderr,
+			"              ignoring: %s...\n",
+			*pBuf);
+	    }
+	    **pBuf = '\0';
+	    return;
+	}
     }
-    *to = 0;
+    *to = '\0';
 }
 
-PRIVATE int PassesTest ARGS1(struct MailcapEntry *,mc)
+PRIVATE int PassesTest ARGS1(
+	struct MailcapEntry *,	mc)
 {
     int result;
     char *cmd, TmpFileName[TMPFILE_NAME_SIZE];
@@ -485,36 +539,38 @@ PRIVATE int PassesTest ARGS1(struct MailcapEntry *,mc)
      *  Make sure we have a command
      */
     if (!mc->testcommand)
-        return(1);
+	return(1);
 
     /*
      *  Save overhead of system() calls by faking these. - FM
      */
     if (0 == strcasecomp(mc->testcommand, "test -n \"$DISPLAY\"")) {
-        FREE(mc->testcommand);
-        if (TRACE)
-	    fprintf(stderr,"Testing for XWINDOWS environment.\n");
+	FREE(mc->testcommand);
+	if (TRACE)
+	    fprintf(stderr,
+		    "PassesTest: Testing for XWINDOWS environment.\n");
     	if ((cp = getenv(DISPLAY)) != NULL && *cp != '\0') {
 	    if (TRACE)
-	        fprintf(stderr,"[HTInit] Test passed!\n");
+	        fprintf(stderr,"PassesTest: Test passed!\n");
 	    return(0 == 0);
 	} else {
 	    if (TRACE)
-	        fprintf(stderr,"[HTInit] Test failed!\n");
+	        fprintf(stderr,"PassesTest: Test failed!\n");
 	    return(-1 == 0);
 	}
     }
     if (0 == strcasecomp(mc->testcommand, "test -z \"$DISPLAY\"")) {
-        FREE(mc->testcommand);
-        if (TRACE)
-	    fprintf(stderr,"Testing for NON_XWINDOWS environment.\n");
+	FREE(mc->testcommand);
+	if (TRACE)
+	    fprintf(stderr,
+		    "PassesTest: Testing for NON_XWINDOWS environment.\n");
     	if (!((cp = getenv(DISPLAY)) != NULL && *cp != '\0')) {
 	    if (TRACE)
-	        fprintf(stderr,"[HTInit] Test passed!\n");
+	        fprintf(stderr,"PassesTest: Test passed!\n");
 	    return(0 == 0);
 	} else {
 	    if (TRACE)
-	        fprintf(stderr,"[HTInit] Test failed!\n");
+	        fprintf(stderr,"PassesTest: Test failed!\n");
 	    return(-1 == 0);
 	}
     }
@@ -523,10 +579,11 @@ PRIVATE int PassesTest ARGS1(struct MailcapEntry *,mc)
      *  Why do anything but return success for this one! - FM
      */
     if (0 == strcasecomp(mc->testcommand, "test -n \"$LYNX_VERSION\"")){
-        FREE(mc->testcommand);
-        if (TRACE) {
-	    fprintf(stderr,"Testing for LYNX environment.\n");
-	    fprintf(stderr,"[HTInit] Test passed!\n");
+	FREE(mc->testcommand);
+	if (TRACE) {
+	    fprintf(stderr,
+		    "PassesTest: Testing for LYNX environment.\n");
+	    fprintf(stderr,"PassesTest: Test passed!\n");
 	}
 	return(0 == 0);
     } else
@@ -534,10 +591,11 @@ PRIVATE int PassesTest ARGS1(struct MailcapEntry *,mc)
      *  ... or failure for this one! - FM
      */
     if (0 == strcasecomp(mc->testcommand, "test -z \"$LYNX_VERSION\"")) {
-        FREE(mc->testcommand);
-        if (TRACE) {
-	    fprintf(stderr,"Testing for non-LYNX environment.\n");
-	    fprintf(stderr,"[HTInit] Test failed!\n");
+	FREE(mc->testcommand);
+	if (TRACE) {
+	    fprintf(stderr,
+		    "PassesTest: Testing for non-LYNX environment.\n");
+	    fprintf(stderr,"PassesTest: Test failed!\n");
 	}
 	return(-1 == 0);
     }
@@ -548,10 +606,13 @@ PRIVATE int PassesTest ARGS1(struct MailcapEntry *,mc)
     tempname(TmpFileName, NEW_FILE);
     cmd = (char *)malloc(1024);
     if (!cmd)
-        ExitWithError("Out of memory");
-    BuildCommand(cmd, mc->testcommand, TmpFileName);
+	ExitWithError("Out of memory");
+    BuildCommand(&cmd, 1024,
+		 mc->testcommand,
+		 TmpFileName,
+		 strlen(TmpFileName));
     if (TRACE)
-        fprintf(stderr,"Executing test command: %s\n", cmd);
+	fprintf(stderr,"PassesTest: Executing test command: %s\n", cmd);
     result = system(cmd);
     FREE(cmd);
 
@@ -562,39 +623,46 @@ PRIVATE int PassesTest ARGS1(struct MailcapEntry *,mc)
     FREE(mc->testcommand);
 
     if (TRACE && result)
-	fprintf(stderr,"[HTInit] Test failed!\n");
+	fprintf(stderr,"PassesTest: Test failed!\n");
     else if (TRACE)
-	fprintf(stderr,"[HTInit] Test passed!\n");
-	
+	fprintf(stderr,"PassesTest: Test passed!\n");
+
     return(result == 0);
 }
 
-PRIVATE int ProcessMailcapFile ARGS1(char *,file)
+PRIVATE int ProcessMailcapFile ARGS1(
+	char *,		file)
 {
     struct MailcapEntry mc;
     FILE *fp;
 
     if (TRACE)
-        fprintf (stderr, "Loading types config file '%s'\n", file);
-
+	fprintf(stderr,
+		"ProcessMailcapFile: Loading file '%s'.\n",
+		file);
     if ((fp = fopen(file, "r")) == NULL) {
-        if (TRACE)
-	    fprintf (stderr, "Could not open types config file '%s'\n",file);
+	if (TRACE)
+	    fprintf(stderr,
+		"ProcessMailcapFile: Could not open '%s'.\n",
+		    file);
 	return(-1 == 0);
     }
 
     while (fp && !feof(fp)) {
-        ProcessMailcapEntry(fp, &mc);
+	ProcessMailcapEntry(fp, &mc);
     }
     fclose(fp);
     return(0 == 0);
 }
 
-PRIVATE int ExitWithError ARGS1(char *,txt)
+PRIVATE int ExitWithError ARGS1(
+	char *,		txt)
 {
     if (txt)
-        fprintf(stderr, "metamail: %s\n", txt);
+	fprintf(stderr, "metamail: %s\n", txt);
+#ifndef NOSIGHUP
     (void) signal(SIGHUP, SIG_DFL);
+#endif /* NOSIGHUP */
     (void) signal(SIGTERM, SIG_DFL);
 #ifndef VMS
     (void) signal(SIGINT, SIG_DFL);
@@ -608,7 +676,8 @@ PRIVATE int ExitWithError ARGS1(char *,txt)
 }
 
 
-PRIVATE int HTLoadTypesConfigFile ARGS1(char *,fn)
+PRIVATE int HTLoadTypesConfigFile ARGS1(
+	char *,		fn)
 {
   return ProcessMailcapFile(fn);
 }
@@ -634,11 +703,10 @@ PRIVATE int HTLoadTypesConfigFile ARGS1(char *,fn)
 PUBLIC void HTFileInit NOARGS
 {
     FILE *fp;
-    extern char *global_extension_map;
-    extern char *personal_extension_map;
 
     if (TRACE)
-        fprintf (stderr, "@@@ Using default extension map\n");
+	fprintf(stderr,
+		"HTFileInit: Loading default (HTInit) extension maps.\n");
 
     /* default suffix interpretation */
     HTSetSuffix("*",		"text/plain", "7bit", 1.0);
@@ -675,7 +743,7 @@ PUBLIC void HTFileInit NOARGS
 
     HTSetSuffix(".exe.Z",	"application/x-Comp. Executable",
     							     "binary", 1.0);
-  
+
     HTSetSuffix(".Z",	        "application/UNIX Compressed", "binary", 1.0);
 
     HTSetSuffix(".tar_Z",	"application/UNIX Compr. Tar", "binary", 1.0);
@@ -685,35 +753,35 @@ PUBLIC void HTFileInit NOARGS
     HTSetSuffix("_gz",		"application/GNU Compressed", "binary", 1.0);
     HTSetSuffix(".gz",		"application/GNU Compressed", "binary", 1.0);
 
-    HTSetSuffix(".tar.gz",	"application/GNU Compr. Tar", "binary", 1.0);
-    HTSetSuffix(".tgz",		"application/GNU Compr. Tar", "binary", 1.0);
+    HTSetSuffix5(".tar.gz",	"application/x-tar", "binary", "GNU Compr. Tar", 1.0);
+    HTSetSuffix5(".tgz",	"application/x-tar", "gzip", "GNU Compr. Tar", 1.0);
 
     HTSetSuffix(".src",		"application/x-WAIS-source", "8bit", 1.0);
     HTSetSuffix(".wsrc",	"application/x-WAIS-source", "8bit", 1.0);
 
     HTSetSuffix(".zip",		"application/x-Zip File", "binary", 1.0);
 
-    HTSetSuffix(".uu",		"application/x-UUencoded", "8bit", 1.0); 
+    HTSetSuffix(".uu",		"application/x-UUencoded", "8bit", 1.0);
 
     HTSetSuffix(".hqx",		"application/x-Binhex", "8bit", 1.0);
 
     HTSetSuffix(".o",		"application/x-Prog. Object", "binary", 1.0);
     HTSetSuffix(".a",		"application/x-Prog. Library", "binary", 1.0);
 
-    HTSetSuffix(".oda",		"application/ODA", "binary", 1.0);
+    HTSetSuffix5(".oda",	"application/oda", "binary", "ODA", 1.0);
 
-    HTSetSuffix(".pdf",		"application/PDF", "binary", 1.0);
+    HTSetSuffix5(".pdf",	"application/pdf", "binary", "PDF", 1.0);
 
     HTSetSuffix(".eps",		"application/Postscript", "8bit", 1.0);
     HTSetSuffix(".ai",		"application/Postscript", "8bit", 1.0);
     HTSetSuffix(".ps",		"application/Postscript", "8bit", 1.0);
-  
+
     HTSetSuffix(".rtf",		"application/RTF", "8bit", 1.0);
 
     HTSetSuffix(".dvi",		"application/x-DVI", "8bit", 1.0);
 
     HTSetSuffix(".hdf",		"application/x-HDF", "8bit", 1.0);
-  
+
     HTSetSuffix(".cdf",		"application/x-netcdf", "8bit", 1.0);
     HTSetSuffix(".nc",		"application/x-netcdf", "8bit", 1.0);
 
@@ -778,7 +846,7 @@ PUBLIC void HTFileInit NOARGS
     HTSetSuffix(".sv4cpio",	"application/x-sv4cpio", "binary", 1.0);
     HTSetSuffix(".sv4crc",	"application/x-sv4crc", "binary", 1.0);
 
-    HTSetSuffix(".tar",		"application/x-Tar File", "binary", 1.0);
+    HTSetSuffix5(".tar",	"application/x-tar", "binary", "Tar File", 1.0);
     HTSetSuffix(".ustar",	"application/x-ustar", "binary", 1.0);
 
     HTSetSuffix(".snd",		"audio/basic", "binary", 1.0);
@@ -790,7 +858,7 @@ PUBLIC void HTFileInit NOARGS
     HTSetSuffix(".wav",		"audio/x-wav", "binary", 1.0);
     HTSetSuffix(".midi",	"audio/midi", "binary", 1.0);
     HTSetSuffix(".mod",		"audio/mod", "binary", 1.0);
-  
+
     HTSetSuffix(".gif",		"image/gif", "binary", 1.0);
     HTSetSuffix(".ief",		"image/ief", "binary", 1.0);
     HTSetSuffix(".jfif",	"image/jpeg", "binary", 1.0); /* xtra */
@@ -806,14 +874,14 @@ PUBLIC void HTFileInit NOARGS
     HTSetSuffix(".pbm",		"image/x-portable-bitmap", "binary", 1.0);
     HTSetSuffix(".pgm",		"image/x-portable-graymap", "binary", 1.0);
     HTSetSuffix(".ppm",		"image/x-portable-pixmap", "binary", 1.0);
-    HTSetSuffix(".png",		"image/x-png", "binary", 1.0);
+    HTSetSuffix(".png",		"image/png", "binary", 1.0);
     HTSetSuffix(".rgb",		"image/x-rgb", "binary", 1.0);
     HTSetSuffix(".xbm",		"image/x-xbitmap", "binary", 1.0);
     HTSetSuffix(".xpm",		"image/x-xpixmap", "binary", 1.0);
     HTSetSuffix(".xwd",		"image/x-xwindowdump", "binary", 1.0);
 
     HTSetSuffix(".rtx",		"text/richtext", "8bit", 1.0);
-    HTSetSuffix(".tsv",		"text/tab-separated-values", "8-bit", 1.0);
+    HTSetSuffix(".tsv",		"text/tab-separated-values", "8bit", 1.0);
     HTSetSuffix(".etx",		"text/x-setext", "8bit", 1.0);
 
     HTSetSuffix(".mpg",		"video/mpeg", "binary", 1.0);
@@ -835,55 +903,69 @@ PUBLIC void HTFileInit NOARGS
     HTSetSuffix(".text",	"text/plain", "8bit", 1.0);
     HTSetSuffix(".txt",		"text/plain", "8bit", 1.0);
 
-    HTSetSuffix(".htm",		"text/html", "8bit", 1.0);
     HTSetSuffix(".html3",	"text/html", "8bit", 1.0);
     HTSetSuffix(".ht3",		"text/html", "8bit", 1.0);
+    HTSetSuffix(".phtml",	"text/html", "8bit", 1.0);
     HTSetSuffix(".shtml",	"text/html", "8bit", 1.0);
     HTSetSuffix(".htmlx",	"text/html", "8bit", 1.0);
+    HTSetSuffix(".htm",		"text/html", "8bit", 1.0);
     HTSetSuffix(".html",	"text/html", "8bit", 1.0);
 
     /* These should override the default extensions as necessary. */
     HTLoadExtensionsConfigFile(global_extension_map);
 
     if ((fp = fopen(personal_extension_map,"r")) != NULL) {
-        fclose(fp);
-        /* These should override everything else. */
-        HTLoadExtensionsConfigFile(personal_extension_map);
+	fclose(fp);
+	/* These should override everything else. */
+	HTLoadExtensionsConfigFile(personal_extension_map);
     } else {
-        char buffer[256];
+	char buffer[256];
 #ifdef VMS
-        sprintf(buffer, "sys$login:%s", personal_extension_map);
+	sprintf(buffer, "sys$login:%s", personal_extension_map);
 #else
-        sprintf(buffer, "%s/%s", (Home_Dir() ? Home_Dir() : ""),
+	sprintf(buffer, "%s/%s", (Home_Dir() ? Home_Dir() : ""),
 				  personal_extension_map);
 #endif /* VMS */
-        /* These should override everything else. */
-        HTLoadExtensionsConfigFile(buffer);
+	/* These should override everything else. */
+	HTLoadExtensionsConfigFile(buffer);
     }
 }
 
 
 /* -------------------- Extension config file reading --------------------- */
 
-/* The following is lifted from NCSA httpd 1.0a1, by Rob McCool;
-   NCSA httpd is in the public domain, as is this code. */
+/*
+ *  The following is lifted from NCSA httpd 1.0a1, by Rob McCool;
+ *  NCSA httpd is in the public domain, as is this code.
+ *
+ *  Modified Oct 97 - KW
+ */
 
 #define MAX_STRING_LEN 256
 
-PRIVATE int HTGetLine ARGS3(char *,s, int,n, FILE *,f) 
+PRIVATE int HTGetLine ARGS3(
+	char *,		s,
+	int,		n,
+	FILE *,		f)
 {
-    register int i = 0;
+    register int i = 0, r;
 
     if (!f)
-        return(1);
+	return(1);
 
     while (1) {
-	s[i] = (char)fgetc(f);
+	r = fgetc(f);
+	s[i] = (char)r;
 
-	if (s[i] == CR)
-	    s[i] = fgetc(f);
+	if (s[i] == CR) {
+	    r = fgetc(f);
+	    if (r == LF)
+		s[i] = r;
+	    else if (r != EOF)
+		ungetc(r, f);
+	}
 
-	if ((s[i] == EOF) || (s[i] == LF) || (i == (n-1))) {
+	if ((r == EOF) || (s[i] == LF) || (s[i] == CR) || (i == (n-1))) {
 	    s[i] = '\0';
 	    return (feof(f) ? 1 : 0);
 	}
@@ -891,65 +973,74 @@ PRIVATE int HTGetLine ARGS3(char *,s, int,n, FILE *,f)
     }
 }
 
-PRIVATE void HTGetWord ARGS4(char *,word, char *,line, char ,stop, char ,stop2) 
+PRIVATE void HTGetWord ARGS4(
+	char *,		word,
+	char *,		line,
+	char ,		stop,
+	char ,		stop2)
 {
     int x = 0, y;
 
     for (x = 0; line[x] && line[x] != stop && line[x] != stop2; x++) {
-        word[x] = line[x];
+	word[x] = line[x];
     }
 
     word[x] = '\0';
-    if (line[x]) 
-        ++x;
+    if (line[x])
+	++x;
     y=0;
 
     while ((line[y++] = line[x++]))
-        ;
+	;
 
     return;
 }
 
-PRIVATE int HTLoadExtensionsConfigFile ARGS1(char *,fn)
+PRIVATE int HTLoadExtensionsConfigFile ARGS1(
+	char *,		fn)
 {
     char l[MAX_STRING_LEN],w[MAX_STRING_LEN],*ct;
     FILE *f;
     int x, count = 0;
 
     if (TRACE)
-        fprintf (stderr, "Loading extensions config file '%s'\n", fn);
+	fprintf(stderr,
+		"HTLoadExtensionsConfigFile: Loading file '%s'.\n", fn);
 
     if ((f = fopen(fn,"r")) == NULL) {
-        if (TRACE)
-            fprintf(stderr, "Could not open extensions config file '%s'\n",fn);
+	if (TRACE)
+	    fprintf(stderr,
+		    "HTLoadExtensionsConfigFile: Could not open '%s'.\n", fn);
 	    return count;
     }
 
     while (!(HTGetLine(l,MAX_STRING_LEN,f))) {
-        HTGetWord(w, l, ' ', '\t');
-        if (l[0] == '\0' || w[0] == '#')
-            continue;
-        ct = (char *)malloc(sizeof(char) * (strlen(w) + 1));
-        if (!ct)
-            outofmem(__FILE__, "HTLoadExtensionsConfigFile");
-        strcpy(ct,w);
-        for (x = 0; ct[x]; x++)
-            ct[x] = TOLOWER(ct[x]);
+	HTGetWord(w, l, ' ', '\t');
+	if (l[0] == '\0' || w[0] == '#')
+	    continue;
+	ct = (char *)malloc(sizeof(char) * (strlen(w) + 1));
+	if (!ct)
+	    outofmem(__FILE__, "HTLoadExtensionsConfigFile");
+	strcpy(ct,w);
+	for (x = 0; ct[x]; x++)
+	    ct[x] = TOLOWER(ct[x]);
 
-        while(l[0]) {
-            HTGetWord(w, l, ' ', '\t');
-            if (w[0] && (w[0] != ' ')) {
-                char *ext = (char *)malloc(sizeof(char) * (strlen(w)+1+1));
+	while(l[0]) {
+	    HTGetWord(w, l, ' ', '\t');
+	    if (w[0] && (w[0] != ' ')) {
+		char *ext = (char *)malloc(sizeof(char) * (strlen(w)+1+1));
 	        if (!ct)
 	            outofmem(__FILE__, "HTLoadExtensionsConfigFile");
 
-                for (x = 0; w[x]; x++)
-                    ext[x+1] = TOLOWER(w[x]);
-                ext[0] = '.';
-                ext[strlen(w)+1] = 0;
+		for (x = 0; w[x]; x++)
+		    ext[x+1] = TOLOWER(w[x]);
+		ext[0] = '.';
+		ext[strlen(w)+1] = '\0';
 
-                if (TRACE)
-                    fprintf (stderr, "SETTING SUFFIX '%s' to '%s'\n", ext, ct);
+		if (TRACE) {
+		    fprintf (stderr,
+			     "SETTING SUFFIX '%s' to '%s'.\n", ext, ct);
+		}
 
 	        if (strstr(ct, "tex") != NULL ||
 	            strstr(ct, "postscript") != NULL ||
@@ -959,12 +1050,12 @@ PRIVATE int HTLoadExtensionsConfigFile ARGS1(char *,fn)
 		    HTSetSuffix (ext, ct, "8bit", 1.0);
 	        else
 		    HTSetSuffix (ext, ct, "binary", 1.0);
-                count++;
+		count++;
 
-                FREE(ext);
-            }
-        }
-        FREE(ct);
+		FREE(ext);
+	    }
+	}
+	FREE(ct);
     }
     fclose(f);
 
