@@ -1,11 +1,18 @@
-
 #ifndef LYOPTIONS_H
 #define LYOPTIONS_H
 
+extern BOOLEAN term_options;
+
 extern void options NOPARAMS;
 extern void edit_bookmarks NOPARAMS;
 
-/* values for options */
+/*
+ *  Values for the options menu. - FM
+ *
+ *  L_foo values are the Y coordinates for the menu item.
+ *  B_foo values are the X coordinates for the item's prompt string.
+ *  C_foo values are the X coordinates for the item's value string.
+ */
 #define L_EDITOR	 2
 #define L_DISPLAY	 3
 
@@ -17,10 +24,15 @@ extern void edit_bookmarks NOPARAMS;
 #define L_FTPSTYPE	 5
 #define L_MAIL_ADDRESS	 6
 #define L_SSEARCH	 7
-#define L_CHARSET	 8
-#define L_RAWMODE	 9
-#define L_LANGUAGE	10
-#define L_PREF_CHARSET	11
+#define L_LANGUAGE	 8
+#define L_PREF_CHARSET	 9
+#define L_ASSUME_CHARSET (L_PREF_CHARSET + 1)
+#define L_CHARSET	10
+#define L_RAWMODE	11
+
+#define L_COLOR		L_RAWMODE
+#define B_COLOR		44
+#define C_COLOR		62
 
 #define L_BOOL_A	12
 #define B_VIKEYS	5
@@ -30,7 +42,12 @@ extern void edit_bookmarks NOPARAMS;
 #define B_SHOW_DOTFILES	44
 #define C_SHOW_DOTFILES	62
 
-#define L_SELECT_POPUPS 13
+#define L_BOOL_B	13
+#define B_SELECT_POPUPS	5
+#define C_SELECT_POPUPS	36
+#define B_SHOW_CURSOR	44
+#define C_SHOW_CURSOR	62
+
 #define L_KEYPAD	14 
 #define L_LINEED	15
 
