@@ -8,14 +8,15 @@
 #include "HTList.h"
 #endif /* HTLIST_H */
 
-extern void highlight PARAMS((int flag, int cur));
+extern void highlight PARAMS((int flag, int cur, char *target));
 extern void free_and_clear PARAMS((char **obj));
 extern void collapse_spaces PARAMS((char *string));
 extern void convert_to_spaces PARAMS((char *string, BOOL condense));
 extern char * strip_trailing_slash PARAMS((char * dirname));
-extern void statusline PARAMS((char *text));
+extern void statusline PARAMS((CONST char *text));
 extern void toggle_novice_line NOPARAMS;
-extern void noviceline PARAMS((int more));
+extern void noviceline PARAMS((int more_flag));
+extern void LYFakeZap PARAMS((BOOL set));
 extern int HTCheckForInterrupt NOPARAMS;
 extern BOOLEAN LYisLocalFile PARAMS((char *filename));
 extern BOOLEAN LYisLocalHost PARAMS((char *filename));
@@ -24,6 +25,7 @@ extern void LYAddLocalhostAlias PARAMS((char *alias));
 extern BOOLEAN LYisLocalAlias PARAMS((char *filename));
 extern int LYCheckForProxyURL PARAMS((char *filename));
 extern int is_url PARAMS((char *filename));
+extern BOOLEAN LYCanDoHEAD PARAMS((CONST char *address));
 extern void remove_backslashes PARAMS((char *buf));
 extern char *quote_pathname PARAMS((char *pathname));
 extern BOOLEAN inlocaldomain NOPARAMS;
@@ -49,14 +51,20 @@ extern void Define_VMSLogical PARAMS((
 	char *LogicalName, char *LogicalValue));
 #endif /* VMS */
 extern CONST char *Home_Dir NOPARAMS;
-extern BOOLEAN LYPathOffHomeOK PARAMS((char *fbuffer, int fbuffer_size));
+extern BOOLEAN LYPathOffHomeOK PARAMS((char *fbuffer, size_t fbuffer_size));
 extern void LYAddPathToHome PARAMS((
-	char *fbuffer, int fbuffer_size, char *fname));
-extern time_t LYmktime PARAMS((char *string));
-#ifdef NO_PUTENV
+	char *fbuffer, size_t fbuffer_size, char *fname));
+extern time_t LYmktime PARAMS((char *string, BOOL absolute));
+#if ! HAVE_PUTENV
 extern int putenv PARAMS((CONST char *string));
-#endif /* NO_PUTENV */
+#endif /* HAVE_PUTENV */
 
+FILE *LYNewBinFile PARAMS((char * name));
+FILE *LYNewTxtFile PARAMS((char * name));
+FILE *LYAppendToTxtFile PARAMS((char * name));
+#ifdef UNIX
+extern void LYRelaxFilePermissions PARAMS((CONST char * name));
+#endif
 /*
  *  Whether or not the status line must be shown.
  */
@@ -131,4 +139,7 @@ extern HTList *sug_filenames;
 #define STREQ(a,b) (strcmp(a,b) == 0)
 #define STRNEQ(a,b,c) (strncmp(a,b,c) == 0)
 
+#define HIDE_CHMOD 0600
+#define HIDE_UMASK 0077
+
 #endif /* LYUTILS_H */
