@@ -6,7 +6,7 @@
  */
 
 /*******************************************************************
- * There are three sections to this document
+ * There are four sections to this document:
  *  Section 1.  Things you MUST change or verify
  *	Section 1a)  VMS specific things
  *	Section 1b)  UNIX specific things
@@ -17,26 +17,42 @@
  *  Section 3.  Things you should only change after you have a good
  *              understanding of the program!
  *
+ *  Section 4.  Things you MUST check only if you plan to use Lynx in
+ *              an anonymous account (allow public access to Lynx)!
+ *
  */
 
 #ifndef USERDEFS_H
 #define USERDEFS_H
 
+#ifdef HAVE_CONFIG_H
+#include <lynx_cfg.h>
+#endif
+
 /*******************************************************************
  * Things you must change
- *  Section 1. 
+ *  Section 1.
  */
 
 /*******************************************************************
  * Things you must change  VMS specific
- *  Section 1a). 
+ *  Section 1a).
  */
 #ifdef VMS
 /**************************
  * TEMP_SPACE is where Lynx temporary cache files will be placed.
  * Temporary files are removed automatically as long as nothing
- * goes terribly wrong :)
- * This definition can be overridden at run time by defining a
+ * goes terribly wrong :)  If you include "$USER" in the definition
+ * (e.g., "device:[dir.$USER]"), Lynx will replace the "$USER" with
+ * the username of the account which invoked the Lynx image.  Such
+ * directories should already exist, and have protections/ACLs set
+ * so that only the appropriate user(s) will have read/write access.
+ * On VMS, "sys$scratch:" defaults to "sys$login:" if it has not been
+ * defined externally, or you can use "sys$login:" explicitly here.
+ * If the path has SHELL syntax and includes a tilde (e.g, "~/lynxtmp"),
+ * Lynx will replace the tilde with the full path for the user's home
+ * and convert the result to VMS syntax.
+ * The definition here can be overridden at run time by defining a
  * "LYNX_TEMP_SPACE" VMS logical.
  */
 #define TEMP_SPACE "sys$scratch:"
@@ -76,7 +92,7 @@
 #endif /* LYNX_CFG_FILE */
 
 /**************************
- * The EXTENSION_MAP file allows you to map file suffix's to 
+ * The EXTENSION_MAP file allows you to map file suffix's to
  * mime types.
  * These global and personal files override anything in
  * lynx.cfg or src/HTInit.c
@@ -85,11 +101,11 @@
 #define PERSONAL_EXTENSION_MAP "mime.types"
 
 /**************************
- * The MAILCAP file allows you to map file MIME types to 
+ * The MAILCAP file allows you to map file MIME types to
  * external viewers.
  * These global and personal files override anything in
  * lynx.cfg or src/HTInit.c
- */ 
+ */
 #define GLOBAL_MAILCAP "Lynx_Dir:mailcap"
 #define PERSONAL_MAILCAP ".mailcap"
 
@@ -109,21 +125,30 @@
 #define XLOADIMAGE_COMMAND "xv %s"
 
 /**************************
- * The full path and name of the standard VMS "mail" command.
+ * SYSTEM_MAIL must be defined here to your mail sending command,
+ * and SYSTEM_MAIL_FLAGS to approrpriate qualifiers.  They can be
+ * changed in lynx.cfg.
  *
  * The mail command will be spawned as a subprocess of lynx
- * and used to send replies and error messages.  It must be
- * defined here, and can be changed in lynx.cfg.
- * SYSTEM_MAIL must be able to accept a subject line through
- * the use of the /subject="SUBJECT" option.
+ * and used to send the email, with headers specified in a
+ * temporary file for PMDF.  If you define SYSTEM_MAIL to the
+ * "generic" MAIL utility for VMS, headers cannot be specified
+ * via a header file (and thus may not be included), and the
+ * subject line will be specified by use of the /subject="SUBJECT"
+ * qualifier.
+ *
  * If your mailer uses another syntax, some hacking of the
- * mailmsg() and reply_by_mail() functions in LYMail.c may
- * be required.
+ * mailform(), mailmsg() and reply_by_mail() functions in
+ * LYMail.c, and printfile() function in LYPrint.c, may be
+ * required.
  */
-#define SYSTEM_MAIL "mail"
+#define SYSTEM_MAIL "PMDF SEND"
+#define SYSTEM_MAIL_FLAGS "/headers"
+/* #define SYSTEM_MAIL "MAIL"   */
+/* #define SYSTEM_MAIL_FLAGS "" */
 
 /*************************
- * below is the argument for an sprintf command that will add
+ * Below is the argument for an sprintf command that will add
  * "IN%""ADDRESS""" to the Internet mail address given by the user.
  * It is structured for PMDF's IN%"INTERNET_ADDRESS" scheme.  The %s
  * is replaced with the address given by the user.  If you are using
@@ -181,13 +206,33 @@
  */
 /*#define NO_ANONYMOUS_EMAIL TRUE */
 
+/**************************
+ * LYNX_LSS_FILE is the location and name of the default lynx
+ * character style sheet file.  It is sought and processed at
+ * startup of Lynx only if experimental character style code has
+ * been compiled in, otherwise it will be ignored.  Note that use
+ * of the character style option is _experimental_ AND _unsupported_.
+ * There is no documentation other than a sample lynx.lss file in
+ * the samples subdirectory.  This code probably won't even work on
+ * VMS.  You can define the location and name of this file via an
+ * environment variable, "lynx_lss", which will override the definition
+ * here.  You can use '~' to refer to the user's home directory.  The
+ * -lss command line switch will override these definitions.
+ */
+#ifndef LYNX_LSS_FILE
+#define LYNX_LSS_FILE "Lynx_Dir:lynx.lss"
+#endif /* LYNX_LSS_FILE */
+
 /*******************************************************************
  * Things you must change  UNIX specific
- *  Section 1b). 
+ *  Section 1b).
  */
 #else     /* UNIX */
 
 /**************************
+ * NOTE: This variable is set by the configure scrip; editing changes will
+ * be ignored.
+ *
  * LYNX_CFG_FILE is the location and name of the default lynx
  * global configuration file.  It is sought and processed at
  * startup of Lynx, followed by a seek and processing of a
@@ -201,6 +246,10 @@
  * -cfg command line switch will override these definitions.
  * You can pass the compilation default via the Makefile.
  *
+ * If you are building Lynx using the configure script, you should specify
+ * the default location of the configuration file via that script, since it
+ * also generates the makefile and install-cfg rules.
+ *
  * Note that many implementations of telnetd allow passing of
  * environment variables, which might be used by unscrupulous
  * people to modify the environment in anonymous accounts.  When
@@ -215,12 +264,18 @@
  * override the src/HTInit.c defaults via the mime.types and
  * mailcap files (see the examples in the samples directory).
  */
+#ifndef HAVE_CONFIG_H
 #ifndef LYNX_CFG_FILE
+#ifdef DOSPATH
+#define LYNX_CFG_FILE "./lynx.cfg"
+#else
 #define LYNX_CFG_FILE "/usr/local/lib/lynx.cfg"
+#endif /* DOSPATH */
 #endif /* LYNX_CFG_FILE */
+#endif /* HAVE_CONFIG_H */
 
 /**************************
- * The EXTENSION_MAP file allows you to map file suffix's to 
+ * The EXTENSION_MAP file allows you to map file suffix's to
  * mime types.
  * These global and personal files override anything in
  * lynx.cfg or src/HTInit.c
@@ -229,7 +284,7 @@
 #define PERSONAL_EXTENSION_MAP ".mime.types"
 
 /**************************
- * The MAILCAP file allows you to map file MIME types to 
+ * The MAILCAP file allows you to map file MIME types to
  * external viewers.
  * These global and personal files override anything in
  * lynx.cfg or src/HTInit.c
@@ -271,7 +326,7 @@
 
 /**************************
  * For UNIX systems this should be sendmail
- * sendmail should be in /usr/lib 
+ * sendmail should be in /usr/lib
  *
  * You definitely want sendmail, not mail or elm or something else,
  * except in the case where MMDF is your mail agent.
@@ -279,34 +334,44 @@
  *
  * SYSTEM_MAIL must be defined here.  You can change it in lynx.cfg.
  *
- * If you do not use the recommended mailers, some hacking of
- * the mailmsg() and reply_by_mail() functions in LYMail.c, or
- * interposition of a script, may be required.
+ * SYSTEM_MAIL_FLAGS must be defined here appropriately for your
+ * SYSTEM_MAIL definition.  You can change it in lynx.cfg.
  */
+#ifndef HAVE_CONFIG_H
 #ifdef MMDF
-#define SYSTEM_MAIL "/usr/mmdf/bin/submit" 
+#define SYSTEM_MAIL "/usr/mmdf/bin/submit"
+#define SYSTEM_MAIL_FLAGS "-mlruxto,cc\\*"
 #else
 #if defined(__FreeBSD__) || defined(__NetBSD__) || defined(__bsdi__)
 #define SYSTEM_MAIL "/usr/sbin/sendmail"
-#else /* everthing else: */
-#define SYSTEM_MAIL "/usr/lib/sendmail" 
+#define SYSTEM_MAIL_FLAGS "-t -oi"
+#else /* everything else: */
+#define SYSTEM_MAIL "/usr/lib/sendmail"
+#define SYSTEM_MAIL_FLAGS "-t -oi"
 #endif /* __FreeBSD || __NetBSD__ || __bsdi__ */
 #endif /* MMDF */
+#endif	/* !HAVE_CONFIG_H */
 
 /**************************
- * A place to put temporary files, it's almost always "/tmp/" on
- * UNIX systems
- * This definition can be overridden at run time by setting a
+ * A place to put temporary files, it's almost always in "/tmp/"
+ * for UNIX systems.  If you include "$USER" in the definition
+ * (e.g., "/tmp/$USER"), Lynx will replace the "$USER" with the
+ * username of the account which invoked the Lynx image.  Such
+ * directories should already exist, and have protections/ACLs set
+ * so that only the appropriate user(s) will have read/write access.
+ * If the path includes a tilde (e.g, "~" or "~/lynxtmp"), Lynx will
+ * replace the tilde with the full path for the user's home.
+ * The definition here can be overridden at run time by setting a
  * "LYNX_TEMP_SPACE" environment symbol.
  */
 #define TEMP_SPACE "/tmp/"
 
 /********************************
- * Don't let the user enter his/hers email address when sending a message.
- * Anonymous mail makes it far too easy for a user to spoof someone elses
+ * Don't let the user enter his/her email address when sending a message.
+ * Anonymous mail makes it far too easy for a user to spoof someone else's
  * email address.
  * This requires that your mailer agent put in the From: field for you.
-
+ *
  * The default should be to uncomment this line but there probably are too
  * many mail agents out there that won't do the right thing if there is no
  * From: line.
@@ -343,6 +408,34 @@
  */
 #define LIST_FORMAT "    %p %4l %-8.8o %-8.8g %7s %-12.12d %a"
 
+/*
+ *  If NO_FORCED_CORE_DUMP is set to TRUE, Lynx will not force
+ *  core dumps via abort() calls on fatal errors or assert()
+ *  calls to check potentially fatal errors.  The default defined
+ *  here can be changed in lynx.cfg, and the compilation or
+ *  configuration default can be toggled via the -core command
+ *  line switch.
+ */
+#define NO_FORCED_CORE_DUMP	FALSE
+
+/**************************
+ * LYNX_LSS_FILE is the location and name of the default lynx
+ * character style sheet file.  It is sought and processed at
+ * startup of Lynx only if experimental character style code
+ * has been compiled in, otherwise it will be ignored.  Note
+ * that use of the character style option is _experimental_ AND
+ * _unsupported_.  There is no documentation other than a sample
+ * lynx.lss file in the samples subdirectory.  You also can
+ * define the location and name of this file via environment
+ * variables "LYNX_LSS" or "lynx_lss" which will override the
+ * "LYNX_LSS_FILE" definition here.  You can use '~' in either or
+ * both definitions to refer to the user's home directory.  The
+ * -lss command line switch will override these definitions.
+ */
+#ifndef LYNX_LSS_FILE
+#define LYNX_LSS_FILE "/usr/local/lib/lynx.lss"
+#endif /* LYNX_LSS_FILE */
+
 #endif /* VMS OR UNIX */
 
 /*************************************************************
@@ -353,14 +446,14 @@
 /*****************************
  * STARTFILE is the default file if none is specified in lynx.cfg,
  *  on the command line, or via a WWW_HOME environment variable.
- * 
+ *
  * note: STARTFILE must be a URL.  See the Lynx online help for more
  *       information on URLs
  */
 #define STARTFILE "http://lynx.browser.org/"
 
 /*****************************
- * HELPFILE must be defined as a URL and must have a 
+ * HELPFILE must be defined as a URL and must have a
  * complete path if local:
  * file://localhost/PATH_TO/lynx_help/lynx_help_main.html
  *   Replace PATH_TO with the path to the lynx_help subdirectory
@@ -381,6 +474,25 @@
  */
 #define DEFAULT_INDEX_FILE "http://www.ncsa.uiuc.edu/SDG/Software/Mosaic/MetaIndex.html"
 
+/*****************************
+ * If USE_TRACE_LOG is set FALSE, then when TRACE mode is invoked the
+ * syserr messages will not be directed to a log file named lynx.trace
+ * in the account's HOME directory.  The default defined here can be
+ * toggled via the -tlog command line switch.  Also, it is set FALSE
+ * automatically when Lynx is executed in an anonymous or validation
+ * account (if indicated via the -anonymous or -validate command line
+ * switches, or via the check for the ANONYMOUS_USER, defined below).
+ * When FALSE, the TRACE_LOG command (normally ';') cannot be used to
+ * examine the Lynx Trace Log during the current session.  If left
+ * TRUE, but you wish to use command line piping of stderr to a file
+ * you specify, include the -tlog toggle on the command line.  Note
+ * that once TRACE mode is turned on during a session and stderr is
+ * directed to the log, all stderr messages will continue going to
+ * the log, even if TRACE mode is turned off via the TOGGLE_TRACE
+ * (Control-T) command.
+ */
+#define USE_TRACE_LOG	FALSE
+
 /*******************************
  * If GOTOBUFFER is set to TRUE here or in lynx.cfg the last entered
  * goto URL, if any, will be offered as a default for reuse or editing
@@ -512,30 +624,60 @@
  * CHARACTER_SET defines the default character set, i.e., that assumed
  * to be installed on the user's termimal.  It determines which characters
  * or strings will be used to represent 8-bit character entities within
- * HTML.  New character sets may be defined by modifying the file
- * src/LYCharSets.c in the Lynx source code distribution and recompiling.
+ * HTML.  New character sets may be defined as explained in the README
+ * files of the src/chrtrans directory in the Lynx source code distribution.
  * For Asian (CJK) character sets, it also determines how Kanji code will
  * be handled.  The default defined here can be changed in lynx.cfg, and
  * via the 'o'ptions menu.  The 'o'ptions menu setting will be stored in
  * the user's RC file whenever those settings are saved, and thereafter
  * will be used as the default.  Also see lynx.cfg for information about
  * the -raw switch and LYE_RAW_TOGGLE command.
+ *
  * The default character sets include:
- *    ISO Latin 1
- *    ISO Latin 2
- *    Other ISO Latin
- *    DEC Multinational
- *    IBM PC character set
- *    IBM PC codepage 850
- *    Macintosh (8 bit)
- *    NeXT character set
- *    KOI8-R character set
- *    Chinese
- *    Japanese (EUC)
- *    Japanese (SJIS)
- *    Korean
- *    Taipei (Big5)
- *    7 bit approximations
+ *
+ *   Display Character Set name		MIME name
+ *   ==========================		=========
+ *   7 bit approximations		us-ascii
+ *   Chinese				euc-cn
+ *   DEC Multinational			dec-mcs
+ *   DosArabic (cp864)			cp864
+ *   DosBaltRim (cp775)			cp775
+ *   DosCyrillic (cp866)		cp866
+ *   DosGreek (cp737)			cp737
+ *   DosGreek2 (cp869)			cp869
+ *   DosHebrew (cp862)			cp862
+ *   DosLatin1 (cp850)			cp850
+ *   DosLatin2 (cp852)			cp852
+ *   DosLatinUS (cp437)			cp437
+ *   ISO 8859-10			iso-8859-10
+ *   ISO 8859-5 Cyrillic		iso-8859-5
+ *   ISO 8859-6 Arabic			iso-8859-6
+ *   ISO 8859-7 Greek			iso-8859-7
+ *   ISO 8859-8 Hebrew			iso-8859-8
+ *   ISO 8859-9 (Latin 5)		iso-8859-9
+ *   ISO Latin 1			iso-8859-1
+ *   ISO Latin 2			iso-8859-2
+ *   ISO Latin 3			iso-8859-3
+ *   ISO Latin 4			iso-8859-4
+ *   Japanese (EUC)			euc-jp
+ *   Japanese (SJIS)			shift_jis
+ *   KOI8-R Cyrillic			koi8-r
+ *   Korean				euc-kr
+ *   Macintosh (8 bit)			macintosh
+ *   NeXT character set			next
+ *   RFC 1345 Mnemonic			mnemonic
+ *   RFC 1345 w/o Intro			mnemonic+ascii+0
+ *   Taipei (Big5)			big5
+ *   Transparent			x-transparent
+ *   UNICODE UTF-8			utf-8
+ *   Vietnamese (VISCII)		viscii
+ *   WinArabic (cp1256)			windows-1256
+ *   WinBaltRim (cp1257)		windows-1257
+ *   WinCyrillic (cp1251)		windows-1251
+ *   WinGreek (cp1253)			windows-1253
+ *   WinHebrew (cp1255)			windows-1255
+ *   WinLatin1 (cp1252)			windows-1252
+ *   WinLatin2 (cp1250)			windows-1250
  */
 #define CHARACTER_SET "ISO Latin 1"
 
@@ -553,12 +695,18 @@
 /*****************************
  * PREFERRED_CHARSET specifies the character set in MIME notation (e.g.,
  * "ISO-8859-2", "ISO-8859-5") which Lynx will indicate you prefer in
- * requests to http servers using an Accept-Charsets header.  The value
- * should NOT include "ISO-8859-1" or "US-ASCII", since those values are
- * always assumed by default.  If a file in that character set is available,
- * the server will send it.  Otherwise, the server will send the file in
- * ISO-8859-1 or US-ASCII.  Users can change or eliminate this definition
+ * requests to http servers using an Accept-Charsets header.
+ * This definition can be overriden via lynx.cfg.  Users also can change it
  * via the 'o'ptions menu and save that preference in their RC file.
+ * The value should NOT include "ISO-8859-1" or "US-ASCII", since those
+ * values are always assumed by default.
+ * If a file in that character set is available, the server will send it.
+ * If no Accept-Charset header is present, the default is that any
+ * character set is acceptable. If an Accept-Charset header is present,
+ * and if the server cannot send a response which is acceptable
+ * according to the Accept-Charset header, then the server SHOULD send
+ * an error response with the 406 (not acceptable) status code, though
+ * the sending of an unacceptable response is also allowed. (RFC2068)
  */
 #define PREFERRED_CHARSET ""
 
@@ -668,8 +816,10 @@
  * the MULTIPLE attribute is present in the SELECT start tag, Lynx
  * always will create a vertical list of checkboxes for the OPTIONs.
  *
- * The default defined here can be changed in lynx.cfg, and can be
- * toggled via the -popup command line switch.
+ * The default defined here can be changed in lynx.cfg.  It can be
+ * set and saved via the 'o'ptions menu to override the compilation
+ * and configuration defaults, and the default always can be toggled
+ * via the -popup command line switch.
  */
 #define USE_SELECT_POPUPS TRUE
 
@@ -715,34 +865,28 @@
 #define MESSAGESECS 2
 #define ALERTSECS 3
 
-/*****************************
- * Enter the name of your anonymous account if you have one
- * as ANONYMOUS_USER.  UNIX systems will use a cuserid
- * or get_login call to determine if the current user is
- * the ANONYMOUS_USER.  VMS systems will use getenv("USER").
- *
- * You may use the "-anonymous" option for multiple accounts,
- * or for precautionary reasons in the anonymous account,
- * as well.
- *
- * It is very important to have this correctly defined or include
- * the "-anonymous" command line option for invokations of Lynx
- * in an anonymous account!  If you do not you will be putting 
- * yourself at GREAT security risk!
- *
- * Later on in this file you can specify privileges for the
- * anonymous account.
+/******************************
+ * SHOW_COLOR controls whether the program displays in color by default.
  */
-#define ANONYMOUS_USER ""
+#ifdef COLOR_CURSES
+#define SHOW_COLOR TRUE
+#else
+#define SHOW_COLOR FALSE
+#endif
 
 /******************************
  * SHOW_CURSOR controls whether or not the cursor is hidden or appears
- * over the link.  The default set here can be changed in lynx.cfg,
- * and can be toggled with the -show_cursor command line option.
+ * over the current link, or current option in select popup windows.
  * Showing the cursor is handy if you are a sighted user with a poor
  * terminal that can't do bold and reverse video at the same time or
  * at all.  It also can be useful to blind users, as an alternative
- * or supplement to setting LINKS_ARE_NUMBERED.
+ * or supplement to setting LINKS_AND_FORM_FIELDS_ARE_NUMBERED or
+ * LINKS_ARE_NUMBERED.
+ *
+ * The default defined here can be changed in lynx.cfg.  It can be
+ * set and saved via the 'o'ptions menu to override the compilation
+ * and configuration defaults, and the default always can be toggled
+ * via the -show_cursor command line switch.
  */
 #define SHOW_CURSOR FALSE
 
@@ -751,129 +895,29 @@
  * curses supports line-drawing characters, set to '*' or any other character
  * to not use line-drawing (e.g., '|' for vertical and '-' for horizontal).
  */
-#define BOXVERT '*'
+#ifndef HAVE_CONFIG_H
+#ifdef DOSPATH
+#define BOXVERT 0
+#define BOXHORI 0
+#else
+#define BOXVERT '|'
 /* #define BOXVERT 0 */
-#define BOXHORI '*'
+#define BOXHORI '-'
 /* #define BOXHORI 0 */
+#endif /* DOSPATH */
+#endif	/* !HAVE_CONFIG_H */
 
 /******************************
  * LY_UMLAUT controls the 7-bit expansion of characters with dieresis or
  * umlaut.  If defined, a digraph is displayed, e.g., auml --> ae
  * Otherwise, a single character is displayed,  e.g., auml --> a
+ * Note that this is currently not supported with the chartrans code,
+ * or rather it doesn't have an effect if translations for a display
+ * character set are taken from one of the *.tbl files in src/chrtrans.
+ * One would have to modify the corresponding *.tbl file for this.
  */
 #define LY_UMLAUT
 
-/*******************************
- * set to FALSE if you don't want users of your anonymous account
- * who are calling from inside your local domain
- * to be able to telnet back out
- */
-#define CAN_ANONYMOUS_INSIDE_DOMAIN_TELNET	TRUE
-
-/*******************************
- * set to FALSE if you don't want users of your anonymous
- * account who are calling from outside your
- * local domain to be able to telnet back out
- */
-#define CAN_ANONYMOUS_OUTSIDE_DOMAIN_TELNET	TRUE
-
-/*******************************
- * set to FALSE if you don't want users of your anonymous account
- * who are calling from inside your local domain
- * to be able to read news
- */
-#define CAN_ANONYMOUS_INSIDE_DOMAIN_READ_NEWS	TRUE
-
-/*******************************
- * set to FALSE if you don't want users of your anonymous
- * account who are calling from outside your
- * local domain to be able to read news
- */
-#define CAN_ANONYMOUS_OUTSIDE_DOMAIN_READ_NEWS	FALSE
-
-/*******************************
- * set to FALSE if you don't want users of your anonymous
- * account who are calling from inside your local domain
- * to be able to use ftp
- */
-#define CAN_ANONYMOUS_INSIDE_DOMAIN_FTP		TRUE
-
-/*******************************
- * set to FALSE if you don't want users of your anonymous
- * account who are calling from outside your local domain
- * to be able to use ftp
- */
-#define CAN_ANONYMOUS_OUTSIDE_DOMAIN_FTP	TRUE
-
-/*******************************
- * set to FALSE if you don't want users of your anonymous
- * account who are calling from inside your local domain 
- * to be able to use rlogin
- */
-#define CAN_ANONYMOUS_INSIDE_DOMAIN_RLOGIN	TRUE
-
-/*******************************
- * set to FALSE if you don't want users of your anonymous
- * account who are calling from outside your local domain
- * to be able to use rlogin
- */
-#define CAN_ANONYMOUS_OUTSIDE_DOMAIN_RLOGIN	TRUE
-
-/*******************************
- * set to FALSE if you don't want users of your anonymous
- * account to be able to goto random URLs. (The 'g' command)
- */
-#define CAN_ANONYMOUS_GOTO		TRUE
-
-/*******************************
- * set to FALSE if you don't want users of your anonymous
- * account to be able to goto particular URLs.
- */
-#define CAN_ANONYMOUS_GOTO_CSO		TRUE
-#define CAN_ANONYMOUS_GOTO_FILE		TRUE
-#define CAN_ANONYMOUS_GOTO_FINGER	TRUE
-#define CAN_ANONYMOUS_GOTO_FTP		TRUE
-#define CAN_ANONYMOUS_GOTO_GOPHER	TRUE
-#define CAN_ANONYMOUS_GOTO_HTTP		TRUE
-#define CAN_ANONYMOUS_GOTO_HTTPS	TRUE
-#define CAN_ANONYMOUS_GOTO_LYNXCGI	TRUE
-#define CAN_ANONYMOUS_GOTO_LYNXEXEC	TRUE
-#define CAN_ANONYMOUS_GOTO_LYNXPROG	TRUE
-#define CAN_ANONYMOUS_GOTO_MAILTO	TRUE
-#define CAN_ANONYMOUS_GOTO_NEWS		TRUE
-#define CAN_ANONYMOUS_GOTO_NNTP		TRUE
-#define CAN_ANONYMOUS_GOTO_RLOGIN	TRUE
-#define CAN_ANONYMOUS_GOTO_SNEWS	TRUE
-#define CAN_ANONYMOUS_GOTO_TELNET	TRUE
-#define CAN_ANONYMOUS_GOTO_TN3270	TRUE
-#define CAN_ANONYMOUS_GOTO_WAIS		TRUE
-
-/*******************************
- * set to FALSE if you don't want users of your anonymous
- * account to be able to specify a port in 'g'oto commands
- * for telnet URLs.
- */
-#define CAN_ANONYMOUS_GOTO_TELNET_PORT	FALSE
-
-/*******************************
- * set to FALSE if you don't want users of your anonymous
- * account to be able to jump to URLs (The 'J' command)
- * via the shortcut entries in your JUMPFILE.
- */
-#define CAN_ANONYMOUS_JUMP	TRUE
-
-/*******************************
- * set to FALSE if you don't want users of your anonymous
- * account to be able to mail
- */
-#define CAN_ANONYMOUS_MAIL	TRUE
-
-/*******************************
- * set to FALSE if you don't want users of your anonymous
- * account to be able to print
- */
-#define CAN_ANONYMOUS_PRINT	FALSE
-
 /*******************************
  * Execution links/scripts configuration.
  *
@@ -922,7 +966,7 @@
  * very serious security risk to your system and its
  * users.  If you do define these I suggest that
  * you only allow users to execute files/scripts
- * that reside on your local machine. 
+ * that reside on your local machine.
  *
  * YOU HAVE BEEN WARNED!
  *
@@ -930,13 +974,13 @@
  * also see src/HTInit.c to verify/change the execution
  * script extensions and/or commands.
  */
-/* #define EXEC_LINKS  */ 
-/* #define EXEC_SCRIPTS  */ 
+/* #define EXEC_LINKS  */
+/* #define EXEC_SCRIPTS  */
 
 /**********
  * UNIX:
  * =====
- * CGI script support. Defining LYNXCGI_LINKS allows you to use the 
+ * CGI script support. Defining LYNXCGI_LINKS allows you to use the
  *
  *   lynxcgi:path
  *
@@ -971,7 +1015,7 @@
 #if defined(EXEC_LINKS) || defined(EXEC_SCRIPTS)
 
 /**********
- * if ALLOW_USERS_TO_CHANGE_EXEC_WITHIN_OPTIONS 
+ * if ALLOW_USERS_TO_CHANGE_EXEC_WITHIN_OPTIONS
  * is defined then the user will be able to change
  * the execution status within the options screen.
  */
@@ -995,7 +1039,7 @@
  * EXEC_SCRIPTS defined.
  *
  * The first two settings:
- * LOCAL_EXECUTION_LINKS_ALWAYS_ON 
+ * LOCAL_EXECUTION_LINKS_ALWAYS_ON
  * LOCAL_EXECUTION_LINKS_ON_BUT_NOT_REMOTE
  * specify the DEFAULT setting of the users execution link
  * options, but the user may still change those options.
@@ -1006,7 +1050,7 @@
  * LOCAL_EXECUTION_LINKS_ALWAYS_ON will be FALSE
  * if NEVER_ALLOW_REMOTE_EXEC has been defined.
  *
- * if LOCAL_EXECUTION_LINKS_ALWAYS_OFF_FOR_ANONYMOUS is 
+ * if LOCAL_EXECUTION_LINKS_ALWAYS_OFF_FOR_ANONYMOUS is
  * true all execution links will be disabled when the
  * -anonymous command line option is used.  Anonymous
  * users are not allowed to change the execution options
@@ -1025,7 +1069,7 @@
 #endif /*  defined(EXEC_LINKS) || defined(EXEC_SCRIPTS) */
 
 /*********************************
- *  MAIL_SYSTEM_ERROR_LOGGING will send a message to the owner of 
+ *  MAIL_SYSTEM_ERROR_LOGGING will send a message to the owner of
  *  the information if there is one, every time
  *  that a document cannot be accessed!
  *
@@ -1055,22 +1099,23 @@
  * VI_KEYS can be turned on by the user in the options
  * screen or the .lynxrc file.  This is just the default.
  */
-#define VI_KEYS_ALWAYS_ON           FALSE /* familiar h,j,k, & l */
+#define VI_KEYS_ALWAYS_ON	FALSE /* familiar h,j,k, & l */
 
 /*********************************
  * EMACS_KEYS can be turned on by the user in the options
  * screen or the .lynxrc file.  This is just the default.
  */
-#define EMACS_KEYS_ALWAYS_ON           FALSE /* familiar ^N, ^P, ^F, ^B */
+#define EMACS_KEYS_ALWAYS_ON	FALSE /* familiar ^N, ^P, ^F, ^B */
 
 /*********************************
  * DEFAULT_KEYPAD_MODE specifies whether by default the user
  * has numbers that work like arrows or else numbered links
- * DEFAULT KEYPAD MODE may be set to 
+ * DEFAULT KEYPAD MODE may be set to
+ *	NUMBERS_AS_ARROWS   or
  *	LINKS_ARE_NUMBERED  or
- *	NUMBERS_AS_ARROWS
+ *	LINKS_AND_FORM_FIELDS_ARE_NUMBERED
  */
-#define DEFAULT_KEYPAD_MODE	       NUMBERS_AS_ARROWS
+#define DEFAULT_KEYPAD_MODE	NUMBERS_AS_ARROWS
 
 /********************************
  * The default search.
@@ -1132,12 +1177,13 @@
 #define SUBSTITUTE_UNDERSCORES	FALSE /* Use _underline_ format in dumps */
 
 /********************************
- * If QUIT_DEFAULT_YES is defined then when the QUIT command is entered,
- * any response other than n or N will confirm.  Comment this out if
- * you prefer the more conservative action of requiring an explicit
- * Y or y to confirm.
+ * If QUIT_DEFAULT_YES is defined as TRUE then when the QUIT command
+ * is entered, any response other than n or N will confirm.  Define it
+ * as FALSE if you prefer the more conservative action of requiring an
+ * explicit Y or y to confirm.  The default defined here can be changed
+ * in lynx.cfg.
  */
-#define QUIT_DEFAULT_YES
+#define QUIT_DEFAULT_YES	TRUE
 
 /********************************
  * These definitions specify files created or used in conjunction
@@ -1148,6 +1194,16 @@
 #define TRAVERSE_REJECT_FILE "reject.dat"
 #define TRAVERSE_ERRORS "traverse.errors"
 
+/****************************************************************
+ * The LYMessages_en.h header defines default, English strings
+ * used in statusline prompts, messages, and warnings during
+ * program execution.  See the comments in LYMessages_en.h for
+ * information on translating or customizing them for your site.
+ */
+#ifndef HTTELNET_H
+#include "LYMessages_en.h"
+#endif /* !HTTELNET_H */
+
 
 /****************************************************************
  *   Section 3.   Things that you should not change until you
@@ -1155,17 +1211,11 @@
  */
 
 #define LYNX_NAME "Lynx"
-#define LYNX_VERSION "2.7.1"
-
-/****************************************************************
- * The LYMessages_en.h header defines default, English strings
- * used in statusline prompts, messages, and warnings during
- * program execution.  See the comments in LYMessages_en.h for
- * information on customizing them for your site.
- */
-#ifndef HTTELNET_H
-#include "LYMessages_en.h"
-#endif /* !HTTELNET_H */
+/* The strange-looking comments on the next line tell PRCS to replace
+ * the version definition with the Project Version on checkout. Just
+ * ignore it. - kw */
+/* $Format: "#define LYNX_VERSION \"$ProjectVersion$\""$ */
+#define LYNX_VERSION "2.8rel.2"
 
 #ifndef MAXINT
 #define MAXINT 2147483647	/* max integer */
@@ -1179,17 +1229,69 @@
 #define MAXHIST  1024		/* max links we remember in history */
 #define MAXLINKS 1024		/* max links on one screen */
 
+#ifndef SEARCH_GOAL_LINE
+#define SEARCH_GOAL_LINE 4	/* try to position search target there */
+#endif
+
+#define MAXCHARSETS 60		/* max character sets supported */
+#define MAXCHARSETSP 61		/* always one more than MAXCHARSETS */
+
+#ifdef __DJGPP__
+#define FNAMES_8_3
+#endif
+
+#ifdef FNAMES_8_3
+#define HTML_SUFFIX ".htm"
+#else
+#define HTML_SUFFIX ".html"
+#endif
+
 #ifdef VMS
 /*
 **  Use the VMS port of gzip for uncompressing both .Z and .gz files.
 */
 #define UNCOMPRESS_PATH  "gzip -d"
 #define GZIP_PATH "gzip"
+
 #else
+
+#ifdef DOSPATH
+/*
+**  Following executables may be sought from your PATH at run-time.
+**  To get those programs look for GNU-port stuff elsewhere.
+**  Currently, if compiled with -DUSE_ZLIB (default), you need only "cp"
+**
+**    WINDOWS
+**  ===========
+*/
+#define COMPRESS_PATH   "compress"
+#define UNCOMPRESS_PATH "uncompress"
+#define UUDECODE_PATH   "uudecode"
+#define ZCAT_PATH       "zcat"
+#define GZIP_PATH       "gzip"
+#define INSTALL_PATH    "install"
+#define TAR_PATH        "tar"
+#define TOUCH_PATH      "touch"
+
+/*
+**    WINDOWS/DOS
+**  ===========
+*/
+#define ZIP_PATH        "zip"
+#define UNZIP_PATH      "unzip"
+#define MKDIR_PATH      "mkdir"
+#define MV_PATH         "mv"
+#define RM_PATH         "rm"
+#define COPY_PATH       "cp"
+#define CHMOD_PATH      "chmod"
+
+#else	/* Unix */
+
 /*
 **  Check these paths on Unix!
 **  ==========================
 */
+#ifndef HAVE_CONFIG_H
 #if defined(__FreeBSD__)||defined(__NetBSD__)||defined(__bsdi__)||defined(LINUX)
 /*
 **  FreeBSD, NetBSD, BSDI, or Linux:
@@ -1201,6 +1303,7 @@
 #define	ZCAT_PATH	"/usr/bin/zcat"
 #define	GZIP_PATH	"/usr/bin/gzip"
 #define	INSTALL_PATH	"/usr/bin/install"
+#define	INSTALL_ARGS	"-c"
 #define	TAR_PATH	"/usr/bin/tar"
 #define	TOUCH_PATH	"/usr/bin/touch"
 #else
@@ -1214,9 +1317,12 @@
 #define	ZCAT_PATH	"/usr/local/bin/zcat"
 #define	GZIP_PATH	"/usr/local/bin/gzip"
 #define	INSTALL_PATH	"/bin/install"
+#define	INSTALL_ARGS	"-c"
 #define	TAR_PATH	"/bin/tar"
 #define	TOUCH_PATH	"/bin/touch"
+
 #endif /* __FreeBSD__ || __NetBSD__ || __bsdi__ || LINUX */
+
 /*
 **  All Unix:
 **  =========
@@ -1228,6 +1334,131 @@
 #define	RM_PATH		"/bin/rm"
 #define COPY_PATH	"/bin/cp"
 #define CHMOD_PATH	"/bin/chmod"
+
+#endif /* HAVE_CONFIG_H */
+#endif /* DOSPATH */
 #endif /* VMS */
 
+
+/****************************************************************
+ *  Section 4.  Things you MUST check only if you plan to use Lynx
+ *              in an anonymous account (allow public access to Lynx).
+ *              This section may be skipped by those people building
+ *              Lynx for private use only.
+ *
+ */
+
+/*****************************
+ * Enter the name of your anonymous account if you have one
+ * as ANONYMOUS_USER.  UNIX systems will use a cuserid
+ * or get_login call to determine if the current user is
+ * the ANONYMOUS_USER.  VMS systems will use getenv("USER").
+ *
+ * You may use the "-anonymous" option for multiple accounts,
+ * or for precautionary reasons in the anonymous account, as well.
+ *
+ * Specify privileges for the anonymous account below.
+ *
+ * It is very important to have this correctly defined or include
+ * the "-anonymous" command line option for invocation of Lynx
+ * in an anonymous account!  If you do not you will be putting
+ * yourself at GREAT security risk!
+ */
+#define ANONYMOUS_USER ""
+
+/*******************************
+ * In the following four pairs of defines,
+ * INSIDE_DOMAIN means users connecting from inside your local domain,
+ * OUTSIDE_DOMAIN means users connecting from outside your local domain.
+ *
+ * set to FALSE if you don't want users of your anonymous
+ * account to be able to telnet back out
+ */
+#define CAN_ANONYMOUS_INSIDE_DOMAIN_TELNET	TRUE
+#define CAN_ANONYMOUS_OUTSIDE_DOMAIN_TELNET	FALSE
+
+/*******************************
+ * set to FALSE if you don't want users of your anonymous
+ * account to be able to use ftp
+ */
+#define CAN_ANONYMOUS_INSIDE_DOMAIN_FTP		TRUE
+#define CAN_ANONYMOUS_OUTSIDE_DOMAIN_FTP	FALSE
+
+/*******************************
+ * set to FALSE if you don't want users of your anonymous
+ * account to be able to use rlogin
+ */
+#define CAN_ANONYMOUS_INSIDE_DOMAIN_RLOGIN	TRUE
+#define CAN_ANONYMOUS_OUTSIDE_DOMAIN_RLOGIN	FALSE
+
+/*******************************
+ * set to FALSE if you don't want users of your anonymous
+ * account to be able to read news
+ */
+#define CAN_ANONYMOUS_INSIDE_DOMAIN_READ_NEWS	TRUE
+#define CAN_ANONYMOUS_OUTSIDE_DOMAIN_READ_NEWS	FALSE
+
+/*******************************
+ * set to FALSE if you don't want users of your anonymous
+ * account to be able to goto random URLs. (The 'g' command)
+ */
+#define CAN_ANONYMOUS_GOTO		TRUE
+
+/*******************************
+ * set to FALSE if you don't want users of your anonymous
+ * account to be able to goto particular URLs.
+ */
+#define CAN_ANONYMOUS_GOTO_CSO		FALSE
+#define CAN_ANONYMOUS_GOTO_FILE		FALSE
+#define CAN_ANONYMOUS_GOTO_FINGER	TRUE
+#define CAN_ANONYMOUS_GOTO_FTP		FALSE
+#define CAN_ANONYMOUS_GOTO_GOPHER	FALSE
+#define CAN_ANONYMOUS_GOTO_HTTP		TRUE
+#define CAN_ANONYMOUS_GOTO_HTTPS	FALSE
+#define CAN_ANONYMOUS_GOTO_LYNXCGI	FALSE
+#define CAN_ANONYMOUS_GOTO_LYNXEXEC	FALSE
+#define CAN_ANONYMOUS_GOTO_LYNXPROG	FALSE
+#define CAN_ANONYMOUS_GOTO_MAILTO	TRUE
+#define CAN_ANONYMOUS_GOTO_NEWS		FALSE
+#define CAN_ANONYMOUS_GOTO_NNTP		FALSE
+#define CAN_ANONYMOUS_GOTO_RLOGIN	FALSE
+#define CAN_ANONYMOUS_GOTO_SNEWS	FALSE
+#define CAN_ANONYMOUS_GOTO_TELNET	FALSE
+#define CAN_ANONYMOUS_GOTO_TN3270	FALSE
+#define CAN_ANONYMOUS_GOTO_WAIS		TRUE
+
+/*******************************
+ * set to FALSE if you don't want users of your anonymous
+ * account to be able to specify a port in 'g'oto commands
+ * for telnet URLs.
+ */
+#define CAN_ANONYMOUS_GOTO_TELNET_PORT	FALSE
+
+/*******************************
+ * set to FALSE if you don't want users of your anonymous
+ * account to be able to jump to URLs (The 'J' command)
+ * via the shortcut entries in your JUMPFILE.
+ */
+#define CAN_ANONYMOUS_JUMP	FALSE
+
+/*******************************
+ * set to FALSE if you don't want users of your anonymous
+ * account to be able to mail
+ */
+#define CAN_ANONYMOUS_MAIL	TRUE
+
+/*******************************
+ * set to FALSE if you don't want users of your anonymous
+ * account to be able to print
+ */
+#define CAN_ANONYMOUS_PRINT	FALSE
+
+/*****************************
+ * Be sure you have read about and set defines above in Sections
+ * 1, 2 and 3 that could  affect Lynx in an anonymous account,
+ * especially LOCAL_EXECUTION_LINKS_ALWAYS_OFF_FOR_ANONYMOUS.
+ *
+ * This ends the section specific to anonymous accounts.
+ */
+
 #endif /* USERDEFS_H */
